!slow-roll functions for the higgs inflation potential (Starobinsky model)
!intended to interface the curvaton module
!
!V(phi) = M^4 * [1 - exp(-sqrt(2/3) phi/Mp)]^2
!
!phi = phi/Mp

  module mchisr
    use infprec, only : kp, tolkp, transfert
    use inftools, only : zbrent
    use curvaton, only : MCphiend, MC_efold_primitive, MC_inftraj_sigma, MC_Delta_faciq
    implicit none

    private

    public mchi_potential, mchi_deriv_potential, mchi_deriv_second_potential, &
          mchi_phiend0, mchi_phi_trajectory, mchi_InverseVp_primitive, &
          mchi_deriv_third_potential, mchi_sigma_trajectory, mchi_Delta_faciq

 
  contains

!returns the potential V
    function mchi_potential(phi,Vparam)
      implicit none
      real(kp) :: mchi_potential, M
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      mchi_potential = M**4*(1._kp-exp(-2._kp*phi/(sqrt(6._kp))))**2

    end function mchi_potential

!returns the first derivative of the potential with respect to phi
    function mchi_deriv_potential(phi,Vparam)
      implicit none
      real(kp) :: mchi_deriv_potential, M
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      mchi_deriv_potential = M**4*(2._kp*sqrt(2._kp/3._kp)* &
              exp(-2._kp*sqrt(2._kp/3._kp)*phi)* (-1._kp+exp(sqrt(2._kp/3._kp)*phi)))

    end function mchi_deriv_potential

!returns the second derivative of the potential with respect to phi
    function mchi_deriv_second_potential(phi,Vparam)
      implicit none
      real(kp) :: mchi_deriv_second_potential, M
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      mchi_deriv_second_potential = M**4*(-(4._kp/3._kp)*exp(-2._kp*sqrt(2._kp/3._kp)*phi)* &
                                      (-2._kp+exp(sqrt(2._kp/3._kp)*phi)))

    end function mchi_deriv_second_potential

!returns the third derivative of the potential with respect to phi
    function mchi_deriv_third_potential(phi,Vparam)
      implicit none
      real(kp) :: mchi_deriv_third_potential, M
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      mchi_deriv_third_potential = M**4*(4._kp/3._kp*sqrt(2._kp/3._kp)* &
            exp(-2._kp*sqrt(2._kp/3._kp)*phi)*(-4._kp+exp(sqrt(2._kp/3._kp)*phi)))

    end function mchi_deriv_third_potential

!returns x at the end of inflation, defined as epsilon1=1, when no curvaton is present
    function mchi_phiend0(Vparam)
      implicit none
      real(kp) :: mchi_phiend0
      type(transfert), optional, intent(inout) :: Vparam

      mchi_phiend0 = sqrt(3._kp/2._kp)*log(1._kp+2._kp/sqrt(3._kp))

    end function mchi_phiend0

!returns Integral [1/V'(phi) dphi]
    function mchi_InverseVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mchi_InverseVp_primitive, M
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1

      mchi_InverseVp_primitive = 0.75_kp/M**4*(exp(sqrt(2._kp/3._kp)*phi)+ &
                log(exp(sqrt(2._kp/3._kp)*phi)-1._kp))

    end function mchi_InverseVp_primitive

!returns Integral [V(phi)/V'(phi) dphi]
    function mchi_VoverVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mchi_VoverVp_primitive
      type(transfert), optional, intent(inout) :: Vparam

      mchi_VoverVp_primitive = -0.5_kp*sqrt(1.5_kp)*phi+0.75_kp*exp(sqrt(2._kp/3._kp)*phi)

    end function mchi_VoverVp_primitive

!returns the function one must zbrent to get phi(efolds)
  function find_mchi_phi_trajectory(phi,mchiData)
    implicit none
    real(kp), intent(in) :: phi
    type(transfert), optional, intent(inout) :: mchiData
    real(kp) :: find_mchi_phi_trajectory
    type(transfert) :: Vparam
    real(kp) :: msigma, sigmaend, NplusNuend, sigma

    Vparam%real1 = mchiData%real1! M

    msigma = mchiData%real3
    sigmaend = mchiData%real4
    NplusNuend = mchiData%real5

    sigma = MC_inftraj_sigma(mchi_potential,mchi_deriv_second_potential,mchi_phiend0, &
            mchi_InverseVp_primitive,Vparam,phi,msigma,sigmaend)
    find_mchi_phi_trajectory = MC_efold_primitive(mchi_VoverVp_primitive,Vparam,phi,sigma,msigma) - &
                                NplusNuend

  end function find_mchi_phi_trajectory

!returns phi at bfold=-efolds before the end of inflation, ie N-Nend
  function mchi_phi_trajectory(bfold, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: bfold, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mchi_phi_trajectory
    real(kp), parameter :: tolFind=tolkp
    real(kp) :: mini, maxi, phiend
    type(transfert) :: mchiData

    phiend = MCphiend(mchi_potential,mchi_deriv_second_potential,mchi_phiend0,Vparam,msigma,sigmaend)

    mini = phiend
    maxi = mini/epsilon(1._kp)

    if (mini > maxi) then
         stop 'phimini>phimaxi in mchi_phi_trajectory'
    endif

    mchiData%real1 = Vparam%real1 ! M

    mchiData%real3 = msigma
    mchiData%real4 = sigmaend
    mchiData%real5 = -bfold + MC_efold_primitive(mchi_VoverVp_primitive,Vparam,phiend,sigmaend,msigma)

    mchi_phi_trajectory = zbrent(find_mchi_phi_trajectory,mini,maxi,tolFind,mchiData)

  end function mchi_phi_trajectory

!returns sigmastar once phistar is computed
  function mchi_sigma_trajectory(phistar, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: phistar, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mchi_sigma_trajectory

    mchi_sigma_trajectory = MC_inftraj_sigma(mchi_potential,mchi_deriv_second_potential,mchi_phiend0, &
                mchi_InverseVp_primitive,Vparam,phistar,msigma,sigmaend)

  end function mchi_sigma_trajectory

!returns Delta once phistar and sigmastar computed
  function mchi_Delta_faciq(Vparam, msigma, phistar, sigmastar)
    implicit none
    real(kp), intent(in) :: msigma, phistar, sigmastar
    type(transfert), optional, intent(in) :: Vparam
    real(kp) :: mchi_Delta_faciq

    mchi_Delta_faciq = MC_Delta_faciq(mchi_potential, mchi_deriv_potential, Vparam, msigma,&
                        phistar, sigmastar)

  end function mchi_Delta_faciq







  end module mchisr
