!slow-roll functions for the large field potential
!intended to interface the curvaton module
!
!V(phi) = M^4 phi^p

  module mclfisr
    use infprec, only : kp, tolkp, transfert
    use inftools, only : zbrent
    use curvaton, only : MCphiend, MC_efold_primitive, MC_inftraj_sigma, MC_Delta_faciq
    implicit none

    private

    public mclfi_potential, mclfi_deriv_potential, mclfi_deriv_second_potential, &
          mclfi_phiend0, mclfi_phi_trajectory, mclfi_InverseVp_primitive, &
          mclfi_deriv_third_potential, mclfi_sigma_trajectory, mclfi_Delta_faciq

 
  contains

!returns the potential V
    function mclfi_potential(phi,Vparam)
      implicit none
      real(kp) :: mclfi_potential, M, p
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      p = Vparam%real2
      mclfi_potential = M**4*phi**p

    end function mclfi_potential

!returns the first derivative of the potential with respect to phi
    function mclfi_deriv_potential(phi,Vparam)
      implicit none
      real(kp) :: mclfi_deriv_potential, M, p
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      p = Vparam%real2
      mclfi_deriv_potential = M**4*p*phi**(p-1._kp)

    end function mclfi_deriv_potential

!returns the second derivative of the potential with respect to phi
    function mclfi_deriv_second_potential(phi,Vparam)
      implicit none
      real(kp) :: mclfi_deriv_second_potential, M, p
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      p = Vparam%real2
      mclfi_deriv_second_potential = M**4*p*(p-1._kp)*phi**(p-2._kp)

    end function mclfi_deriv_second_potential

!returns the third derivative of the potential with respect to phi
    function mclfi_deriv_third_potential(phi,Vparam)
      implicit none
      real(kp) :: mclfi_deriv_third_potential, M, p
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      p = Vparam%real2
      mclfi_deriv_third_potential = M**4*p*(p-1._kp)*(p-2._kp)*phi**(p-3._kp)

    end function mclfi_deriv_third_potential

!returns x at the end of inflation, defined as epsilon1=1, when no curvaton is present
    function mclfi_phiend0(Vparam)
      implicit none
      real(kp) :: mclfi_phiend0, p
      type(transfert), optional, intent(inout) :: Vparam

      p = Vparam%real2
      mclfi_phiend0 = p/sqrt(2._kp)

    end function mclfi_phiend0

!returns Integral [1/V'(phi) dphi]
    function mclfi_InverseVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mclfi_InverseVp_primitive, M, p
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      p = Vparam%real2
      if (p == 2._kp) then
        mclfi_InverseVp_primitive = 0.5_kp/M**4*log(phi)
      else
        mclfi_InverseVp_primitive = 1/(p*(2._kp-p)*M**4)*phi**(2._kp-p)
      end if

    end function mclfi_InverseVp_primitive

!returns Integral [V(phi)/V'(phi) dphi]
    function mclfi_VoverVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mclfi_VoverVp_primitive, p
      type(transfert), optional, intent(inout) :: Vparam

      p = Vparam%real2
      mclfi_VoverVp_primitive = 0.5_kp*phi**2/p

    end function mclfi_VoverVp_primitive

!returns the function one must zbrent to get phi(efolds)
  function find_mclfi_phi_trajectory(phi,mclfiData)
    implicit none
    real(kp), intent(in) :: phi
    type(transfert), optional, intent(inout) :: mclfiData
    real(kp) :: find_mclfi_phi_trajectory
    type(transfert) :: Vparam
    real(kp) :: msigma, sigmaend, NplusNuend, sigma

    Vparam%real1 = mclfiData%real1! M
    Vparam%real2 = mclfiData%real2 ! p

    msigma = mclfiData%real3
    sigmaend = mclfiData%real4
    NplusNuend = mclfiData%real5

    sigma = MC_inftraj_sigma(mclfi_potential,mclfi_deriv_second_potential,mclfi_phiend0, &
            mclfi_InverseVp_primitive,Vparam,phi,msigma,sigmaend)
    find_mclfi_phi_trajectory = MC_efold_primitive(mclfi_VoverVp_primitive,Vparam,phi,sigma,msigma) - NplusNuend

  end function find_mclfi_phi_trajectory

!returns phi at bfold=-efolds before the end of inflation, ie N-Nend
  function mclfi_phi_trajectory(bfold, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: bfold, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mclfi_phi_trajectory
    real(kp), parameter :: tolFind=tolkp
    real(kp) :: mini, maxi, phiend
    type(transfert) :: mclfiData

    phiend = MCphiend(mclfi_potential,mclfi_deriv_second_potential,mclfi_phiend0,Vparam,msigma,sigmaend)

    mini = phiend
    maxi = mini/epsilon(1._kp)

    if (mini > maxi) then
         stop 'phimini>phimaxi in mclfi_phi_trajectory'
    endif

    mclfiData%real1 = Vparam%real1 ! M
    mclfiData%real2 = Vparam%real2 ! p
    mclfiData%real3 = msigma
    mclfiData%real4 = sigmaend
    mclfiData%real5 = -bfold + MC_efold_primitive(mclfi_VoverVp_primitive,Vparam,phiend,sigmaend,msigma)

    mclfi_phi_trajectory = zbrent(find_mclfi_phi_trajectory,mini,maxi,tolFind,mclfiData)

  end function mclfi_phi_trajectory

!returns sigmastar once phistar is computed
  function mclfi_sigma_trajectory(phistar, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: phistar, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mclfi_sigma_trajectory

    mclfi_sigma_trajectory = MC_inftraj_sigma(mclfi_potential,mclfi_deriv_second_potential,mclfi_phiend0, &
                mclfi_InverseVp_primitive,Vparam,phistar,msigma,sigmaend)

  end function mclfi_sigma_trajectory

!returns Delta once phistar and sigmastar computed
  function mclfi_Delta_faciq(Vparam, msigma, phistar, sigmastar)
    implicit none
    real(kp), intent(in) :: msigma, phistar, sigmastar
    type(transfert), optional, intent(in) :: Vparam
    real(kp) :: mclfi_Delta_faciq

    mclfi_Delta_faciq = MC_Delta_faciq(mclfi_potential, mclfi_deriv_potential, Vparam, msigma,&
                        phistar, sigmastar)

  end function mclfi_Delta_faciq




  
  end module mclfisr
