!slow-roll functions for the natural inflation potential
!intended to interface the curvaton module
!
!V(phi) = M^4 [ 1 + cos(phi/f) ]
!
!phi = phi/Mp
!f = f/Mp


  module mcnisr
    use infprec, only : kp, tolkp, transfert
    use inftools, only : zbrent
    use curvaton, only : MCphiend, MC_efold_primitive, MC_inftraj_sigma, MC_Delta_faciq
    implicit none

    private

    public mcni_potential, mcni_deriv_potential, mcni_deriv_second_potential, &
          mcni_phiend0, mcni_phi_trajectory, mcni_InverseVp_primitive, &
          mcni_deriv_third_potential, mcni_sigma_trajectory, mcni_Delta_faciq

 
  contains

!returns the potential V
    function mcni_potential(phi,Vparam)
      implicit none
      real(kp) :: mcni_potential, M, f
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      f = Vparam%real2
      mcni_potential = M**4*(1._kp+cos(phi/f))

    end function mcni_potential

!returns the first derivative of the potential with respect to phi
    function mcni_deriv_potential(phi,Vparam)
      implicit none
      real(kp) :: mcni_deriv_potential, M, f
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      f = Vparam%real2
      mcni_deriv_potential = -M**4*sin(phi/f)/f

    end function mcni_deriv_potential

!returns the second derivative of the potential with respect to phi
    function mcni_deriv_second_potential(phi,Vparam)
      implicit none
      real(kp) :: mcni_deriv_second_potential, M, f
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      f = Vparam%real2
      mcni_deriv_second_potential = -M**4*cos(phi/f)/(f**2)

    end function mcni_deriv_second_potential

!returns the third derivative of the potential with respect to phi
    function mcni_deriv_third_potential(phi,Vparam)
      implicit none
      real(kp) :: mcni_deriv_third_potential, M, f
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      f = Vparam%real2
      mcni_deriv_third_potential = M**4*sin(phi/f)/(f**3)

    end function mcni_deriv_third_potential

!returns phi at the end of inflation, defined as epsilon1=1, when no curvaton is present
    function mcni_phiend0(Vparam)
      implicit none
      real(kp) :: mcni_phiend0, f
      type(transfert), optional, intent(inout) :: Vparam

      f = Vparam%real2
      mcni_phiend0 = f*acos((1._kp-2._kp*f**2)/(1._kp+2._kp*f**2))

    end function mcni_phiend0

!returns Integral [1/V'(phi) dphi]
    function mcni_InverseVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mcni_InverseVp_primitive, M, f
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      f = Vparam%real2

      mcni_InverseVp_primitive = -log(abs(tan(0.5_kp*phi)))/M**4

    end function mcni_InverseVp_primitive

!returns Integral [V(phi)/V'(phi) dphi]
    function mcni_VoverVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mcni_VoverVp_primitive, f
      type(transfert), optional, intent(inout) :: Vparam

      f = Vparam%real2
      mcni_VoverVp_primitive = -f**2*log(abs( &
         1._kp-cos(phi/f)))

    end function mcni_VoverVp_primitive

!returns the function one must zbrent to get phi(efolds)
  function find_mcni_phi_trajectory(phi,mcniData)
    implicit none
    real(kp), intent(in) :: phi
    type(transfert), optional, intent(inout) :: mcniData
    real(kp) :: find_mcni_phi_trajectory
    type(transfert) :: Vparam
    real(kp) :: msigma, sigmaend, NplusNuend, sigma

    Vparam%real1 = mcniData%real1! M
    Vparam%real2 = mcniData%real2 ! f

    msigma = mcniData%real3
    sigmaend = mcniData%real4
    NplusNuend = mcniData%real5

    sigma = MC_inftraj_sigma(mcni_potential,mcni_deriv_second_potential,mcni_phiend0, &
            mcni_InverseVp_primitive,Vparam,phi,msigma,sigmaend)
    find_mcni_phi_trajectory = MC_efold_primitive(mcni_VoverVp_primitive,Vparam,phi,sigma,msigma) - NplusNuend

  end function find_mcni_phi_trajectory

!returns phi at bfold=-efolds before the end of inflation, ie N-Nend
  function mcni_phi_trajectory(bfold, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: bfold, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mcni_phi_trajectory
    real(kp), parameter :: tonind=tolkp
    real(kp) :: mini, maxi, phiend
    type(transfert) :: mcniData

    phiend = MCphiend(mcni_potential,mcni_deriv_second_potential,mcni_phiend0,Vparam,msigma,sigmaend)

    maxi = phiend
    mini = epsilon(1._kp)

    if (mini > maxi) then
         stop 'phimini>phimaxi in mcni_phi_trajectory'
    endif

    mcniData%real1 = Vparam%real1 ! M
    mcniData%real2 = Vparam%real2 ! f
    mcniData%real3 = msigma
    mcniData%real4 = sigmaend
    mcniData%real5 = -bfold + MC_efold_primitive(mcni_VoverVp_primitive,Vparam,phiend,sigmaend,msigma)

    mcni_phi_trajectory = zbrent(find_mcni_phi_trajectory,mini,maxi,tonind,mcniData)

  end function mcni_phi_trajectory

!returns sigmastar once phistar is computed
  function mcni_sigma_trajectory(phistar, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: phistar, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mcni_sigma_trajectory

    mcni_sigma_trajectory = MC_inftraj_sigma(mcni_potential,mcni_deriv_second_potential,mcni_phiend0, &
                mcni_InverseVp_primitive,Vparam,phistar,msigma,sigmaend)

  end function mcni_sigma_trajectory

!returns Delta once phistar and sigmastar computed
  function mcni_Delta_faciq(Vparam, msigma, phistar, sigmastar)
    implicit none
    real(kp), intent(in) :: msigma, phistar, sigmastar
    type(transfert), optional, intent(in) :: Vparam
    real(kp) :: mcni_Delta_faciq

    mcni_Delta_faciq = MC_Delta_faciq(mcni_potential, mcni_deriv_potential, Vparam, msigma,&
                        phistar, sigmastar)

  end function mcni_Delta_faciq


  
  end module mcnisr
