!slow-roll functions for the power law inflation potential
!intended to interface the curvaton module
!
!V(phi) = M^4 Exp(-alpha phi/Mp)
!
!phi = phi/Mp
!alpha


  module mcplisr
    use infprec, only : kp, tolkp, transfert
    use inftools, only : zbrent
    use curvaton, only : MCphiend, MC_efold_primitive, MC_inftraj_sigma, MC_Delta_faciq
    implicit none

    private

    real(kp), parameter :: junkphiend = 100._kp

    public mcpli_potential, mcpli_deriv_potential, mcpli_deriv_second_potential, &
          mcpli_phiend0, mcpli_phi_trajectory, mcpli_InverseVp_primitive, &
          mcpli_deriv_third_potential, mcpli_sigma_trajectory, mcpli_Delta_faciq

 
  contains

!returns the potential V
    function mcpli_potential(phi,Vparam)
      implicit none
      real(kp) :: mcpli_potential, M, alpha
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      alpha = Vparam%real2
      mcpli_potential = M**4*exp(-alpha*phi)

    end function mcpli_potential

!returns the first derivative of the potential with respect to phi
    function mcpli_deriv_potential(phi,Vparam)
      implicit none
      real(kp) :: mcpli_deriv_potential, M, alpha
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      alpha = Vparam%real2
      mcpli_deriv_potential = -M**4*alpha*exp(-alpha*phi)

    end function mcpli_deriv_potential

!returns the second derivative of the potential with respect to phi
    function mcpli_deriv_second_potential(phi,Vparam)
      implicit none
      real(kp) :: mcpli_deriv_second_potential, M, alpha
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      alpha = Vparam%real2
      mcpli_deriv_second_potential = M**4*alpha**2*exp(-alpha*phi)

    end function mcpli_deriv_second_potential

!returns the third derivative of the potential with respect to phi
    function mcpli_deriv_third_potential(phi,Vparam)
      implicit none
      real(kp) :: mcpli_deriv_third_potential, M, alpha
      real(kp), intent(in) :: phi
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      alpha = Vparam%real2
      mcpli_deriv_third_potential = -M**4*alpha**3*exp(-alpha*phi)

    end function mcpli_deriv_third_potential

!returns phi at the end of inflation, defined as epsilon1=1, when no curvaton is present
    function mcpli_phiend0(Vparam)
      implicit none
      real(kp) :: mcpli_phiend0, alpha
      type(transfert), optional, intent(inout) :: Vparam

      alpha = Vparam%real2
      mcpli_phiend0 = junkphiend

    end function mcpli_phiend0

!returns Integral [1/V'(phi) dphi]
    function mcpli_InverseVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mcpli_InverseVp_primitive, M, alpha
      type(transfert), optional, intent(inout) :: Vparam

      M = Vparam%real1
      alpha = Vparam%real2

      mcpli_InverseVp_primitive = -exp(alpha*phi)/(M**4*alpha**2)

    end function mcpli_InverseVp_primitive

!returns Integral [V(phi)/V'(phi) dphi]
    function mcpli_VoverVp_primitive(phi,Vparam)
      implicit none
      real(kp), intent (in) :: phi
      real(kp) :: mcpli_VoverVp_primitive, alpha
      type(transfert), optional, intent(inout) :: Vparam

      alpha = Vparam%real2
      mcpli_VoverVp_primitive = -phi/alpha

    end function mcpli_VoverVp_primitive

!returns phi at bfold=-efolds before the end of inflation, ie N-Nend
  function mcpli_phi_trajectory(bfold, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: bfold, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mcpli_phi_trajectory
    real(kp) :: alpha, phiend

    alpha = Vparam%real2
    phiend = MCphiend(mcpli_potential,mcpli_deriv_second_potential,mcpli_phiend0,Vparam,msigma,sigmaend)

    mcpli_phi_trajectory = phiend+alpha*bfold

  end function mcpli_phi_trajectory

!returns sigmastar once phistar is computed
  function mcpli_sigma_trajectory(phistar, Vparam, msigma, sigmaend)
    implicit none
    real(kp), intent(in) :: phistar, msigma, sigmaend
    type(transfert), optional, intent(inout) :: Vparam
    real(kp) :: mcpli_sigma_trajectory

    mcpli_sigma_trajectory = MC_inftraj_sigma(mcpli_potential,mcpli_deriv_second_potential,mcpli_phiend0, &
                mcpli_InverseVp_primitive,Vparam,phistar,msigma,sigmaend)

  end function mcpli_sigma_trajectory

!returns Delta once phistar and sigmastar computed
  function mcpli_Delta_faciq(Vparam, msigma, phistar, sigmastar)
    implicit none
    real(kp), intent(in) :: msigma, phistar, sigmastar
    type(transfert), optional, intent(in) :: Vparam
    real(kp) :: mcpli_Delta_faciq

    mcpli_Delta_faciq = MC_Delta_faciq(mcpli_potential, mcpli_deriv_potential, Vparam, msigma,&
                        phistar, sigmastar)

  end function mcpli_Delta_faciq

  
  end module mcplisr
