module curvaton

! Functions relevant for the massive curvaton (MC) calculations

  use infprec, only : kp, tolkp, transfert, pi, CConst
  use specialinf, only : lambert
  use srreheat, only : pi, Nzero
  use cosmopar, only : powerAmpScalar
  use inftools, only : zbrent

  implicit none

  private

  public MCphiend, MC_efold_primitive, MC_inftraj_sigma, find_MC_bfoldstar, MC_Delta, &
         MC_r, MC_ns, MC_alphaS, MC_Delta_faciq
  public MC1_lnRrad, MC1_Pzeta, MC1_fNL_phistarfree, MC1_sigmaend_max
  public MC2_lnRrad, MC2_Pzeta, MC2_fNL_phistarfree, MC2_sigmaend_min, MC2_sigmaend_max
  public MC3_lnRrad, MC3_Pzeta, MC3_fNL_phistarfree, MC3_sigmaend_min, MC3_sigmaend_max, &
         MC3_Mmin_reh
  public MC4_lnRrad, MC4_Pzeta, MC4_fNL_phistarfree, MC4_sigmaend_max
  public MC5_lnRrad, MC5_Pzeta, MC5_fNL_phistarfree, MC5_sigmaend_min, MC5_sigmaend_max
  public MC6_lnRrad, MC6_Pzeta, MC6_fNL_phistarfree, MC6_sigmaend_min, MC6_sigmaend_max, &
         MC6_Mmin_reh
  public MC7_lnRrad, MC7_Pzeta, MC7_fNL_phistarfree, MC7_sigmaend_max
  public MC8_lnRrad, MC8_Pzeta, MC8_fNL_phistarfree, MC8_sigmaend_min, MC8_sigmaend_max
  public MC9_lnRrad, MC9_Pzeta, MC9_fNL_phistarfree, MC9_sigmaend_min, MC9_sigmaend_max, &
         MC9_Mmin_reh
  public MC10_lnRrad, MC10_Pzeta, MC10_fNL_phistarfree, MC10_sigmaend_min, MC10_sigmaend_max, &
         MC10_Mmin_reh
  public MC1_Pzeta_relativecontribution_sigma, MC2_Pzeta_relativecontribution_sigma, &
         MC3_Pzeta_relativecontribution_sigma, MC4_Pzeta_relativecontribution_sigma, &
         MC5_Pzeta_relativecontribution_sigma, MC6_Pzeta_relativecontribution_sigma, &
         MC7_Pzeta_relativecontribution_sigma, MC8_Pzeta_relativecontribution_sigma, &
         MC9_Pzeta_relativecontribution_sigma, MC10_Pzeta_relativecontribution_sigma


  contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                        !!
!!        INFLATION       !!
!!                        !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  SLOW ROLL PARAMETERS  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns epsilon1 in the phi direction
  function MC_epsilon_one_phi(V,Vp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_epsilon_one_phi
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
    end interface

    MC_epsilon_one_phi = 0.5_kp*(Vp(phi,Vparam)**2)/ &
                         (V(phi,Vparam)+0.5_kp*msigma**2*sigma**2)**2

  end function MC_epsilon_one_phi

! Returns epsilon1 in the sigma direction
  function MC_epsilon_one_sigma(V,Vp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_epsilon_one_sigma
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
    end interface

    MC_epsilon_one_sigma = 0.5_kp*(msigma**4*sigma**2)/ &
                         (V(phi,Vparam)+0.5_kp*msigma**2*sigma**2)**2

  end function MC_epsilon_one_sigma

! Returns epsilon1
  function MC_epsilon_one(V,Vp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_epsilon_one
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
    end interface

    MC_epsilon_one = 0.5_kp*(Vp(phi,Vparam)**2+msigma**4*sigma**2)/ &
                         (V(phi,Vparam)+0.5_kp*msigma**2*sigma**2)**2

  end function MC_epsilon_one

! Returns the thrajectory angle in field space
  function MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_theta
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
    end interface

    MC_theta = atan(sqrt(MC_epsilon_one_sigma(V,Vp,Vparam,phi,sigma,msigma)/ &
                MC_epsilon_one_phi(V,Vp,Vparam,phi,sigma,msigma)))

  end function MC_theta

! Returns eta in the phi phi direction
  function MC_eta_phiphi(V,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_phiphi
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    MC_eta_phiphi= Vpp(phi,Vparam)/(V(phi,Vparam)+0.5_kp*msigma**2*sigma**2)

  end function MC_eta_phiphi

! Returns eta in the sigma sigma direction
  function MC_eta_sigmasigma(V,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_sigmasigma
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    MC_eta_sigmasigma= msigma**2/(V(phi,Vparam)+0.5_kp*msigma**2*sigma**2)

  end function MC_eta_sigmasigma

! Returns eta in the phi sigma direction
  function MC_eta_phisigma(V,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_phisigma
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    MC_eta_phisigma = 0._kp

  end function MC_eta_phisigma

! Returns eta in the curvature direction
  function MC_eta_cc(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_cc, etaphiphi, etaphisigma, etasigmasigma, theta
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    etaphiphi = MC_eta_phiphi(V,Vpp,Vparam,phi,sigma,msigma)
    etaphisigma = MC_eta_phisigma(V,Vpp,Vparam,phi,sigma,msigma)
    etasigmasigma = MC_eta_sigmasigma(V,Vpp,Vparam,phi,sigma,msigma)
    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    MC_eta_cc = etaphiphi*cos(theta)**2+2._kp*cos(theta)*sin(theta)*etaphisigma+ &
                 sin(theta)**2*etasigmasigma

  end function MC_eta_cc

! Returns eta in the isocurvature direction
  function MC_eta_ss(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_ss, etaphiphi, etaphisigma, etasigmasigma, theta
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    etaphiphi = MC_eta_phiphi(V,Vpp,Vparam,phi,sigma,msigma)
    etaphisigma = MC_eta_phisigma(V,Vpp,Vparam,phi,sigma,msigma)
    etasigmasigma = MC_eta_sigmasigma(V,Vpp,Vparam,phi,sigma,msigma)
    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    MC_eta_ss = etaphiphi*sin(theta)**2-2._kp*cos(theta)*sin(theta)*etaphisigma+ &
                cos(theta)**2*etasigmasigma

  end function MC_eta_ss

! Returns eta in the curvature - isocurvature direction
  function MC_eta_cs(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_eta_cs, etaphiphi, etaphisigma, etasigmasigma, theta
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    etaphiphi = MC_eta_phiphi(V,Vpp,Vparam,phi,sigma,msigma)
    etaphisigma = MC_eta_phisigma(V,Vpp,Vparam,phi,sigma,msigma)
    etasigmasigma = MC_eta_sigmasigma(V,Vpp,Vparam,phi,sigma,msigma)
    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    MC_eta_cs = cos(theta)*sin(theta)*(etasigmasigma-etaphiphi)+ &
                 etaphisigma*(cos(theta)**2-sin(theta)**2)

  end function MC_eta_cs

! Returns the xi parameter in the curvature direction
! See Eq (A7) of astroph/0605679
  function MC_xi_ccc(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_xi_ccc, theta, Vval, Vpval, Vpppval
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    Vval = V(phi, Vparam)
    Vpval = Vp(phi, Vparam)
    Vpppval = Vppp(phi, Vparam)
    MC_xi_ccc = cos(theta)**3*Vpppval*(cos(theta)*Vpval+sin(theta)*msigma**2*sigma)/ &
                (Vval+0.5_kp*msigma**2*sigma**2)**2

  end function MC_xi_ccc

! Returns the xi parameter in the curvature-curvature-isocurvature direction
! See Eq (A7) of astroph/0605679
  function MC_xi_ccs(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_xi_ccs, theta, Vval, Vpval, Vpppval
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    Vval = V(phi, Vparam)
    Vpval = Vp(phi, Vparam)
    Vpppval = Vppp(phi, Vparam)
    MC_xi_ccs = -cos(theta)**2*sin(theta)*Vpppval*(cos(theta)*Vpval+sin(theta)*msigma**2*sigma)/ &
                (Vval+0.5_kp*msigma**2*sigma**2)**2

  end function MC_xi_ccs

! Returns the xi parameter in the curvature-isocurvature-isocurvature direction
! See Eq (A7) of astroph/0605679
  function MC_xi_css(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_xi_css, theta, Vval, Vpval, Vpppval
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    Vval = V(phi, Vparam)
    Vpval = Vp(phi, Vparam)
    Vpppval = Vppp(phi, Vparam)
    MC_xi_css = cos(theta)*sin(theta)**2*Vpppval*(cos(theta)*Vpval+sin(theta)*msigma**2*sigma)/ &
                (Vval+0.5_kp*msigma**2*sigma**2)**2

  end function MC_xi_css

! Returns the xi parameter in the isocurvature direction
! See Eq (A7) of astroph/0605679
  function MC_xi_sss(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_xi_sss, theta, Vval, Vpval, Vpppval
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    theta = MC_theta(V,Vp,Vparam,phi,sigma,msigma)
    Vval = V(phi, Vparam)
    Vpval = Vp(phi, Vparam)
    Vpppval = Vppp(phi, Vparam)
    MC_xi_sss = sin(theta)**3*Vpppval*(cos(theta)*Vpval+sin(theta)*msigma**2*sigma)/ &
                (Vval+0.5_kp*msigma**2*sigma**2)**2

  end function MC_xi_sss

! Returns spectral index of curvature fluctuations, see Eq(40) of astroph/0205253 and Eq(63) of astroph/0605679 for second order expressions
  function MC_ns(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma,Delta)
    implicit none
    real(kp) :: MC_ns, etacc, etacs, etass, epsilon, xiccc, xiccs, xicss, nsSecondOrder
    real(kp), intent(in) :: phi, sigma, msigma, Delta
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    etacc = MC_eta_cc(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    etacs = MC_eta_cs(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    etass = MC_eta_ss(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    xiccc = MC_xi_ccc(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    xiccs = MC_xi_ccs(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    xicss = MC_xi_css(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    epsilon = MC_epsilon_one(V,Vp,Vparam,phi,sigma,msigma)

    MC_ns = 1._kp-(6._kp-4._kp*cos(Delta)**2)*epsilon+2._kp*(etacc*sin(Delta)**2+ &
             2._kp*etacs*sin(Delta)*cos(Delta)+etass*cos(Delta)**2)

    ! Second order contribution
    nsSecondOrder = (-10._kp/3._kp-4._kp*cos(Delta)**2-CConst*(24._kp-16._kp*cos(Delta)**2)) &
             *epsilon**2+2._kp/3._kp*sin(Delta)**2*etacc**2+(2._kp/3._kp*cos(Delta)**2 &
             -4._kp*CConst*(1._kp-2._kp*cos(Delta)**2))*etacs**2+2._kp/3._kp* &
             cos(Delta)**2*etass**2+(-2._kp+14._kp/3._kp*cos(Delta)**2-CConst*&
             (-16._kp+12._kp*cos(Delta)**2))*epsilon*etacc-4._kp/3._kp*sin(Delta)* &
             cos(Delta)*(1._kp-6._kp*CConst)*epsilon*etacs-2._kp/3._kp*cos(Delta)**2 &
             *(-1._kp-6._kp*CConst)*epsilon*etass+4._kp/3._kp*sin(Delta)*cos(Delta)* &
             (1._kp+3._kp*CConst)*etacc*etacs+4._kp/3._kp*sin(Delta)*cos(Delta)* &
             (1._kp-3._kp*CConst)*etacs*etass+2./3._kp*sin(Delta)**2*(1._kp-3._kp* &
             CConst)*xiccc+4._kp/3._kp*sin(Delta)*cos(Delta)*(1._kp-3._kp*CConst)* &
             xiccs+2._kp/3._kp*cos(Delta)**2*(1._kp-3._kp*CConst)*xicss

   !MC_ns = MC_ns + nsSecondOrder

  end function MC_ns

! Returns tensor-to-scalar ratio, see Eq(38) of asrtoph/0205253 and Eq(82) of astroph/0605679 for a second order expression
  function MC_r(V,Vp,Vpp,Vparam,phi,sigma,msigma,Delta)
    implicit none
    real(kp) :: MC_r, epsilon, etacc
    real(kp), intent(in) :: phi, sigma, msigma, Delta
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
    end interface

    etacc = MC_eta_cc(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    epsilon = MC_epsilon_one(V,Vp,Vparam,phi,sigma,msigma)

    MC_r = 16._kp*epsilon*sin(Delta)**2*(1._kp-(4._kp/3._kp-4._kp*CConst)*epsilon+ &
            (2._kp/3._kp-2._kp*CConst)*etacc)

  end function MC_r


! Returns scalar running, see Eq(66) of astroph/0605679
  function MC_alphaS(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma,Delta)
    implicit none
    real(kp) :: MC_alphaS, etacc, etacs, etass, epsilon, xiccc, xiccs, xicss, &
                alphaSecondOrder
    real(kp), intent(in) :: phi, sigma, msigma, Delta
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function Vppp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vppp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vppp
    end interface

    etacc = MC_eta_cc(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    etacs = MC_eta_cs(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    etass = MC_eta_ss(V,Vp,Vpp,Vparam,phi,sigma,msigma)
    xiccc = MC_xi_ccc(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    xiccs = MC_xi_ccs(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    xicss = MC_xi_css(V,Vp,Vpp,Vppp,Vparam,phi,sigma,msigma)
    epsilon = MC_epsilon_one(V,Vp,Vparam,phi,sigma,msigma)

    MC_alphaS = 0._kp

    ! Second order contribution
    alphaSecondOrder = 8._kp*(-3._kp+4._kp*cos(Delta)**2-2._kp*cos(Delta)**4)* &
                epsilon**2+4._kp*sin(Delta)**2*cos(Delta)**2*etass**2+4._kp* &
                (1._kp-4._kp*sin(Delta)**2*cos(Delta)**2)*etacs**2+4._kp*sin(Delta)**2* &
                cos(Delta)**2*etacc**2+4._kp*(4._kp-7._kp*cos(Delta)**2+ &
                4._kp*cos(Delta)**4)*epsilon*etass+32._kp*sin(Delta)**3*cos(Delta)* &
                epsilon*etacs+4._kp*cos(Delta)**2*(5._kp-4._kp*cos(Delta)**2)*epsilon* &
                etacc-8._kp*sin(Delta)*cos(Delta)*(1._kp-2._kp*cos(Delta)**2)*etass* &
                etacs-8._kp*sin(Delta)**2*cos(Delta)**2*etacc*etass+8._kp*sin(Delta)* &
                cos(Delta)*(1._kp-2._kp*cos(Delta)**2)*etacs*etass-2._kp*sin(Delta)**2* &
                xiccc**2-4._kp*sin(Delta)*cos(Delta)*xiccs**2-2._kp*cos(Delta)**2* &
                xicss**2

 ! MC_alphaS=MC_alphaS+alphaSecondOrder

  end function MC_alphaS

!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  SLOW ROLL TRAJECTORY  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns sigma(phi)
  function MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phi,msigma,sigmaend)
    implicit none
    real(kp) :: MC_inftraj_sigma, phiend
    real(kp), intent(in) :: phi,msigma,sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface


    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC_inftraj_sigma = sigmaend*exp(msigma**2* &
                       (InverseVp_primitive(phi,Vparam)-InverseVp_primitive(phiend,Vparam)))

  end function MC_inftraj_sigma

!Returns the number of efold realized at (phi, sigma)
function MC_efold_primitive(VoverVp_primitive,Vparam,phi,sigma,msigma)
    implicit none
    real(kp) :: MC_efold_primitive, phiend
    real(kp), intent(in) :: phi,sigma,msigma
    type(transfert) :: Vparam
    interface
        function VoverVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: VoverVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function VoverVp_primitive
    end interface

    MC_efold_primitive = VoverVp_primitive(phi,Vparam)+0.25_kp*sigma**2

  end function MC_efold_primitive


!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                        !!
!!    END OF INFLATION    !!
!!                        !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the value of the inflaton field at the end of inflation
  function MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    implicit none
    real(kp), intent(in) :: msigma,sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface
    real(kp) :: MCphiend,phiend0val,Vend0, Vppend0

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MCphiend = phiend0val+msigma**2*sigmaend**2/(sqrt(2._kp)*Vend0)/(Vppend0/Vend0-2._kp)

  end function MCphiend

! Returns the value of the Hubble parameter at the end of inflation
  function MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    implicit none
    real(kp), intent(in) :: msigma,sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface
    real(kp) :: MC_Hend,phiend0val,Vend0, Vppend0

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_Hend = sqrt(0.5_kp*Vend0)*(1._kp+0.5_kp*msigma**2*sigmaend**2/Vend0* &
            (0.5_kp+1._kp/(Vppend0/Vend0-2._kp)))

  end function MC_Hend

! Returns the value of the energy density contained in sigma at the end of inflation
  function MC_rhosigma_end(V,phiend0,Vparam,msigma,sigmaend)
    implicit none
    real(kp), intent(in) :: msigma,sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface
    real(kp) :: MC_rhosigma_end,phiend0val,Vend0

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    MC_rhosigma_end = 0.5_kp*msigma**2*sigmaend**2*(1._kp+2._kp/9._kp*msigma**2/Vend0)

  end function MC_rhosigma_end

! Returns the value of the energy density contained in phi at the end of inflation
  function MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    implicit none
    real(kp), intent(in) :: msigma,sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface
    real(kp) :: MC_rhophi_end,phiend0val,Vend0, Vppend0

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_rhophi_end =  1.5_kp*Vend0+msigma**2*sigmaend**2*(0.25_kp+1.5_kp/(Vppend0/Vend0-2._kp))

  end function MC_rhophi_end

! Returns the maximum value of sigmaend such that it is subdominant in eps1_end
  function MC_sigmaendmax_epsilon_one(V,Vp,phiend0,Vparam,msigma)
    implicit none
    real(kp) :: MC_sigmaendmax_epsilon_one, phiend0val,Vend0, Vpend0
    real(kp), intent(in) ::msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vpend0 = Vp(phiend0val,Vparam)
    MC_sigmaendmax_epsilon_one = min(abs(Vpend0)/msigma**2,sqrt(2._kp*Vend0)/msigma)

  end function MC_sigmaendmax_epsilon_one

!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                      !!
!!   End of Inflation   !!
!!     perturbed by     !!
!!   phi* and sigma*    !!
!!                      !!
!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns d phi_end / d sigma_end
  function MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    implicit none
    real(kp) :: MC_dphiend_dsigmaend, phiend0val, Vend0, Vppend0
    real(kp), intent(in) :: sigmaend, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_dphiend_dsigmaend = sqrt(2._kp)*msigma**2*sigmaend/Vend0/(Vppend0/Vend0-2._kp)

  end function MC_dphiend_dsigmaend

!Returns d^2 phi_end / d sigma_end^2
  function MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    implicit none
    real(kp) :: MC_ddphiend_ddsigmaend, phiend0val, Vend0, Vppend0
    real(kp), intent(in) :: sigmaend, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_ddphiend_ddsigmaend = sqrt(2._kp)*msigma**2/Vend0/(Vppend0/Vend0-2._kp)

  end function MC_ddphiend_ddsigmaend

!Returns d H_end / d sigma_end
  function MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    implicit none
    real(kp) :: MC_dHend_dsigmaend, phiend0val, Vend0, Vppend0
    real(kp), intent(in) :: sigmaend, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_dHend_dsigmaend = msigma**2*sigmaend/sqrt(2._kp*Vend0)*(0.5_kp+1._kp/(Vppend0/Vend0-2._kp))

  end function MC_dHend_dsigmaend

!Returns d^2 H_end / d sigma_end ^2
  function MC_ddHend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    implicit none
    real(kp) :: MC_ddHend_ddsigmaend, phiend0val, Vend0, Vppend0
    real(kp), intent(in) :: sigmaend, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam)
    Vppend0 = Vpp(phiend0val,Vparam)
    MC_ddHend_ddsigmaend = msigma**2/sqrt(2._kp*Vend0)*(0.5_kp+1._kp/(Vppend0/Vend0-2._kp))

  end function MC_ddHend_ddsigmaend

!Returns d sigmaend / d sigmastar
  function MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dsigmaend_dsigmastar, sigmastar, Vpend, phiend, dphienddsigmaend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vpend = Vp(phiend,Vparam)
    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)

    MC_dsigmaend_dsigmastar = 1._kp/(sigmastar/sigmaend-msigma**2*sigmastar*dphienddsigmaend/Vpend)

  end function MC_dsigmaend_dsigmastar

!Returns d^2 sigmaend / d sigmastar^2
  function MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddsigmaend_ddsigmastar, sigmastar, Vpend, Vppend, phiend, dphienddsigmaend, &
               ddphiendddsigmaend, dsigmaenddsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)

    MC_ddsigmaend_ddsigmastar = dsigmaenddsigmastar**2*(sigmastar/sigmaend**2*dsigmaenddsigmastar+ &
       2._kp*msigma**2*sigmastar*dphienddsigmaend/Vpend-msigma**2*sigmastar**2*Vppend/Vpend**2* &
       dphienddsigmaend**2*dsigmaenddsigmastar+msigma**2*sigmastar**2/Vpend*ddphiendddsigmaend* &
       dsigmaenddsigmastar-1._kp/sigmaend)

  end function MC_ddsigmaend_ddsigmastar

!Returns d sigmaend / d phistar
  function MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dsigmaend_dphistar, Vpstar, Vpend, phiend, dphienddsigmaend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vpend = Vp(phiend,Vparam)
    Vpstar = Vp(phistar,Vparam)
    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)

    MC_dsigmaend_dphistar = 1._kp/(Vpstar/Vpend*dphienddsigmaend-Vpstar/(msigma**2*sigmaend))

  end function MC_dsigmaend_dphistar

!Returns d^2 sigmaend / d phistar^2
  function MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddsigmaend_ddphistar, Vpend, Vppend, phiend, Vpstar, Vppstar, dphienddsigmaend, &
               ddphiendddsigmaend, dsigmaenddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    Vpstar = Vp(phistar,Vparam)
    Vppstar = Vpp(phistar,Vparam)
    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)

    MC_ddsigmaend_ddphistar = dsigmaenddphistar**2*(Vpstar/Vpend**2*Vppend*dphienddsigmaend**2* &
       dsigmaenddphistar-Vpstar/Vpend*ddphiendddsigmaend*dsigmaenddphistar+Vppstar/ &
       (msigma**2*sigmaend)-Vpstar/(msigma**2*sigmaend)*dsigmaenddphistar-Vppstar/ &
       Vpend*dphienddsigmaend)

  end function MC_ddsigmaend_ddphistar

!Returns d^2 sigmaend / d phistar d sigmastar
  function MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddsigmaend_dphistar_dsigmastar, Vpend, Vppend, phiend, Vpstar, dphienddsigmaend, &
               ddphiendddsigmaend, dsigmaenddsigmastar, dsigmaenddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    Vpstar = Vp(phistar,Vparam)
    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)

    MC_ddsigmaend_dphistar_dsigmastar = dsigmaenddphistar**2*(Vpstar/Vpend**2*Vppend* &
        dphienddsigmaend**2*dsigmaenddsigmastar-Vpstar/Vpend*ddphiendddsigmaend*dsigmaenddsigmastar &
        -Vpstar/(msigma**2*sigmaend**2)*dsigmaenddsigmastar)

  end function MC_ddsigmaend_dphistar_dsigmastar

!Returns d H_end / d sigmastar
  function MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    implicit none
    real(kp) :: MC_dHend_dsigmastar, dHenddsigmaend, dsigmaenddsigmastar
    real(kp), intent(in) :: sigmaend, msigma, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dHenddsigmaend = MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    MC_dHend_dsigmastar = dHenddsigmaend*dsigmaenddsigmastar

  end function MC_dHend_dsigmastar

!Returns d^2 H_end / d sigmastar^2
  function MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    implicit none
    real(kp) :: MC_ddHend_ddsigmastar, dHenddsigmaend, ddHendddsigmaend, dsigmaenddsigmastar, &
               ddsigmaendddsigmastar
    real(kp), intent(in) :: sigmaend, msigma, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dHenddsigmaend = MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddHendddsigmaend = MC_ddHend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                          Vparam, sigmaend, phistar, msigma)
    MC_ddHend_ddsigmastar = ddHendddsigmaend*dsigmaenddsigmastar**2+ &
                            dHenddsigmaend*ddsigmaendddsigmastar

  end function MC_ddHend_ddsigmastar

!Returns d H_end / d phistar
  function MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    implicit none
    real(kp) :: MC_dHend_dphistar, dHenddsigmaend, dsigmaenddphistar
    real(kp), intent(in) :: sigmaend, msigma, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dHenddsigmaend = MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    MC_dHend_dphistar = dHenddsigmaend*dsigmaenddphistar

  end function MC_dHend_dphistar

!Returns d^2 H_end / d phistar^2
  function MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    implicit none
    real(kp) :: MC_ddHend_ddphistar, dHenddsigmaend, ddHendddsigmaend, dsigmaenddphistar, &
               ddsigmaendddphistar
    real(kp), intent(in) :: sigmaend, msigma, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dHenddsigmaend = MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddHendddsigmaend = MC_ddHend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                          Vparam, sigmaend, phistar, msigma)
    MC_ddHend_ddphistar = ddHendddsigmaend*dsigmaenddphistar**2+ &
                            dHenddsigmaend*ddsigmaendddphistar

  end function MC_ddHend_ddphistar

!Returns d^2 H_end / d phistar d sigmastar
  function MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    implicit none
    real(kp) :: MC_ddHend_dphistar_dsigmastar, dHenddsigmaend, ddHendddsigmaend, dsigmaenddphistar, &
               dsigmaenddsigmastar, ddsigmaenddphistardsigmastar
    real(kp), intent(in) :: sigmaend, msigma, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dHenddsigmaend = MC_dHend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddHendddsigmaend = MC_ddHend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
                         InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    MC_ddHend_dphistar_dsigmastar = ddHendddsigmaend*dsigmaenddphistar*dsigmaenddsigmastar+ &
                            dHenddsigmaend*ddsigmaenddphistardsigmastar

  end function MC_ddHend_dphistar_dsigmastar

!Returns d phiend / d sigmastar
  function MC_dphiend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dphiend_dsigmastar, dphienddsigmaend, dsigmaenddsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_dphiend_dsigmastar = dphienddsigmaend*dsigmaenddsigmastar

  end function MC_dphiend_dsigmastar

!Returns d phiend / d phistar
  function MC_dphiend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dphiend_dphistar, dphienddsigmaend, dsigmaenddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_dphiend_dphistar = dphienddsigmaend*dsigmaenddphistar

  end function MC_dphiend_dphistar

!Returns d^2 phiend / d phistar^2
  function MC_ddphiend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddphiend_ddphistar, dphienddsigmaend, dsigmaenddphistar, ddphiendddsigmaend, &
                ddsigmaendddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddphiend_ddphistar = dsigmaenddphistar**2*ddphiendddsigmaend+ &
                             dphienddsigmaend*ddsigmaendddphistar

  end function MC_ddphiend_ddphistar

!Returns d^2 phiend / d sigmastar^2
  function MC_ddphiend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddphiend_ddsigmastar, dphienddsigmaend, dsigmaenddsigmastar, ddphiendddsigmaend, &
                ddsigmaendddsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddphiend_ddsigmastar = dsigmaenddsigmastar**2*ddphiendddsigmaend+ &
                               dphienddsigmaend*ddsigmaendddsigmastar

  end function MC_ddphiend_ddsigmastar

!Returns d^2 phiend / d sigmastar d phistar
  function MC_ddphiend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddphiend_dphistar_dsigmastar, dphienddsigmaend, dsigmaenddsigmastar, &
               dsigmaenddphistar, ddphiendddsigmaend, ddsigmaenddphistardsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dphienddsigmaend = MC_dphiend_dsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    ddphiendddsigmaend = MC_ddphiend_ddsigmaend(V, Vpp, phiend0, Vparam, msigma, sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddphiend_dphistar_dsigmastar = dsigmaenddphistar*dsigmaenddsigmastar*ddphiendddsigmaend+ &
                                       dphienddsigmaend*ddsigmaenddphistardsigmastar

  end function MC_ddphiend_dphistar_dsigmastar

!Returns d Nend / d sigmastar
  function MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dNend_dsigmastar, sigmastar, dphienddsigmastar, dsigmaenddsigmastar, Vend, Vpend, phiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vpend = Vp(phiend,Vparam)
    dphienddsigmastar = MC_dphiend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_dNend_dsigmastar = 0.5_kp*sigmastar-0.5_kp*sigmaend*dsigmaenddsigmastar- &
                          Vend/Vpend*dphienddsigmastar

  end function MC_dNend_dsigmastar

!Returns d Nend / d phistar
  function MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dNend_dphistar, dphienddphistar, dsigmaenddphistar, Vend, Vpend, phiend, Vstar, &
               Vpstar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vpend = Vp(phiend,Vparam)
    Vstar = V(phistar,Vparam)
    Vpstar = Vp(phistar,Vparam)
    dphienddphistar = MC_dphiend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_dNend_dphistar = Vstar/Vpstar-dphienddphistar*Vend/Vpend-0.5_kp*sigmaend*dsigmaenddphistar

  end function MC_dNend_dphistar

!Returns d^2 Nend / dsigmatar^2
  function MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddNend_ddsigmastar, dphienddsigmastar, ddsigmaendddsigmastar, dsigmaenddsigmastar,&
                ddphiendddsigmastar, Vend, Vpend, Vppend, phiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dphienddsigmastar = MC_dphiend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddphiendddsigmastar = MC_ddphiend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)

    MC_ddNend_ddsigmastar = 0.5_kp-0.5_kp*dsigmaenddsigmastar**2-(1._kp-Vend*Vppend/Vpend**2)* &
       dphienddsigmastar**2-0.5_kp*sigmaend*ddsigmaendddsigmastar-Vend/Vpend*ddphiendddsigmastar

  end function MC_ddNend_ddsigmastar

!Returns d^2 Nend / d phistar^2
  function MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddNend_ddphistar, dphienddphistar, ddsigmaendddphistar, dsigmaenddphistar,&
                ddphiendddphistar, Vend, Vpend, Vppend, phiend, Vstar, Vpstar, Vppstar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    Vstar = V(phistar,Vparam)
    Vpstar = Vp(phistar,Vparam)
    Vppstar = Vpp(phistar,Vparam)
    dphienddphistar = MC_dphiend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddphiendddphistar = MC_ddphiend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)

    MC_ddNend_ddphistar = 1._kp-Vstar*Vppstar/Vpstar**2-ddphiendddphistar*Vend/Vpend-0.5_kp* &
      dsigmaenddphistar**2-dphienddphistar**2*(1._kp-Vend*Vppend/Vpend**2)-0.5_kp*sigmaend* &
      ddsigmaendddphistar

  end function MC_ddNend_ddphistar

!Returns d^2 Nend / d phistar d sigmastar
  function MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddNend_dphistar_dsigmastar, dphienddphistar, dphienddsigmastar, &
               ddsigmaenddphistardsigmastar, dsigmaenddphistar, dsigmaenddsigmastar, &
                ddphienddphistardsigmastar, Vend, Vpend, Vppend, phiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vpend = Vp(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dphienddphistar = MC_dphiend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dphienddsigmastar = MC_dphiend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddphienddphistardsigmastar = MC_ddphiend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddNend_dphistar_dsigmastar = -0.5_kp*(2._kp*dphienddsigmastar*dphienddphistar*(1._kp-Vend* &
        Vppend/Vpend**2)+dsigmaenddsigmastar*dsigmaenddphistar+2._kp*Vend/Vpend* &
        ddphienddphistardsigmastar+sigmaend*ddsigmaenddphistardsigmastar)

  end function MC_ddNend_dphistar_dsigmastar

! Returns d ln rhosigma_end / d sigmastar
  function MC_dlnrho_sigma_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dlnrho_sigma_end_dsigmastar, dsigmaenddsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_dlnrho_sigma_end_dsigmastar = 2._kp/sigmaend*dsigmaenddsigmastar

  end function MC_dlnrho_sigma_end_dsigmastar

! Returns d ln rhosigma_end / d phistar
  function MC_dlnrho_sigma_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dlnrho_sigma_end_dphistar, dsigmaenddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_dlnrho_sigma_end_dphistar = 2._kp/sigmaend*dsigmaenddphistar

  end function MC_dlnrho_sigma_end_dphistar

! Returns d^2 ln rhosigma_end / d sigmastar^2
  function MC_ddlnrho_sigma_end_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_sigma_end_ddsigmastar, dsigmaenddsigmastar, ddsigmaendddsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_ddlnrho_sigma_end_ddsigmastar = 2._kp/sigmaend*ddsigmaendddsigmastar- &
                                       2._kp/sigmaend**2*dsigmaenddsigmastar**2

  end function MC_ddlnrho_sigma_end_ddsigmastar

! Returns d^2 ln rhosigma_end / d phistar^2
  function MC_ddlnrho_sigma_end_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_sigma_end_ddphistar, dsigmaenddphistar, ddsigmaendddphistar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_ddlnrho_sigma_end_ddphistar = 2._kp/sigmaend*ddsigmaendddphistar- &
                                       2._kp/sigmaend**2*dsigmaenddphistar**2

  end function MC_ddlnrho_sigma_end_ddphistar

! Returns d^2 ln rhosigma_end / d phistar d sigmastar
  function MC_ddlnrho_sigma_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_sigma_end_dphistar_dsigmastar, dsigmaenddsigmastar, dsigmaenddphistar, &
                ddsigmaenddphistardsigmastar
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddlnrho_sigma_end_dphistar_dsigmastar = 2._kp/sigmaend*ddsigmaenddphistardsigmastar- &
                                       2._kp/sigmaend**2*dsigmaenddsigmastar*dsigmaenddphistar

  end function MC_ddlnrho_sigma_end_dphistar_dsigmastar

! Returns d rhophi_end / d sigmastar
  function MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_drho_phi_end_dsigmastar, dsigmaenddsigmastar, phiend, Vend, Vppend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_drho_phi_end_dsigmastar = 2._kp*msigma**2*sigmaend*dsigmaenddsigmastar*(0.25_kp+1.5_kp/(Vppend/Vend-2._kp))

  end function MC_drho_phi_end_dsigmastar

! Returns d lnrhophi_end / d sigmastar
  function MC_dlnrho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dlnrho_phi_end_dsigmastar, drhophienddsigmastar, rhophiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC_dlnrho_phi_end_dsigmastar = drhophienddsigmastar/rhophiend

  end function MC_dlnrho_phi_end_dsigmastar

! Returns d^2 rhophi_end / d sigmastar^2
  function MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddrho_phi_end_ddsigmastar, dsigmaenddsigmastar, ddsigmaendddsigmastar, phiend, &
                Vend, Vppend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)

    MC_ddrho_phi_end_ddsigmastar = 2._kp*msigma**2*(dsigmaenddsigmastar**2+sigmaend* &
          ddsigmaendddsigmastar)*(0.25_kp+1.5_kp/(Vppend/Vend-2._kp))

  end function MC_ddrho_phi_end_ddsigmastar

! Returns d^2 lnrhophi_end / d sigmastar^2
  function MC_ddlnrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_phi_end_ddsigmastar, drhophienddsigmastar, ddrhophiendddsigmastar, &
               rhophiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC_ddlnrho_phi_end_ddsigmastar = ddrhophiendddsigmastar/rhophiend- &
        (drhophienddsigmastar/rhophiend)**2

  end function MC_ddlnrho_phi_end_ddsigmastar

! Returns d rhophi_end / d phistar
  function MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_drho_phi_end_dphistar, dsigmaenddphistar, phiend, Vend, Vppend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC_drho_phi_end_dphistar = 2._kp*msigma**2*sigmaend*dsigmaenddphistar*(0.25_kp+1.5_kp/(Vppend/Vend-2._kp))

  end function MC_drho_phi_end_dphistar

! Returns d lnrhophi_end / d phistar
  function MC_dlnrho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_dlnrho_phi_end_dphistar, drhophienddphistar, rhophiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC_dlnrho_phi_end_dphistar = drhophienddphistar/rhophiend

  end function MC_dlnrho_phi_end_dphistar

! Returns d^2 rhophi_end / d phistar^2
  function MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddrho_phi_end_ddphistar, dsigmaenddphistar, ddsigmaendddphistar, phiend, &
                Vend, Vppend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)

    MC_ddrho_phi_end_ddphistar = 2._kp*msigma**2*(dsigmaenddphistar**2+sigmaend* &
          ddsigmaendddphistar)*(0.25_kp+1.5_kp/(Vppend/Vend-2._kp))

  end function MC_ddrho_phi_end_ddphistar

! Returns d^2 lnrhophi_end / d phistar^2
  function MC_ddlnrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_phi_end_ddphistar, drhophienddphistar, ddrhophiendddphistar, &
               rhophiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC_ddlnrho_phi_end_ddphistar = ddrhophiendddphistar/rhophiend- &
        (drhophienddphistar/rhophiend)**2

  end function MC_ddlnrho_phi_end_ddphistar

! Returns d^2 rhophi_end / d phistar d sigmastar
  function MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
           sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddrho_phi_end_dphistar_dsigmastar, dsigmaenddphistar, dsigmaenddsigmastar, &
                ddsigmaenddphistardsigmastar, phiend, Vend, Vppend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Vend = V(phiend,Vparam)
    Vppend = Vpp(phiend,Vparam)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC_ddrho_phi_end_dphistar_dsigmastar = 2._kp*msigma**2*(dsigmaenddphistar*dsigmaenddsigmastar+ &
          sigmaend*ddsigmaenddphistardsigmastar)*(0.25_kp+1.5_kp/(Vppend/Vend-2._kp))

  end function MC_ddrho_phi_end_dphistar_dsigmastar

! Returns d^2 lnrhophi_end / d phistar d sigmastar
  function MC_ddlnrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    implicit none
    real(kp) :: MC_ddlnrho_phi_end_dphistar_dsigmastar, drhophienddphistar, drhophienddsigmastar, &
               ddrhophienddphistardsigmastar, rhophiend
    real(kp), intent(in) :: sigmaend, phistar, msigma
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC_ddlnrho_phi_end_dphistar_dsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
        drhophienddphistar*drhophienddsigmastar/rhophiend**2

  end function MC_ddlnrho_phi_end_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

  function MC_rdec(RhoGammaDec,RhoMatterDec)
    implicit none
    real(kp) :: MC_rdec
    real(kp), intent(in) :: RhoGammaDec,RhoMatterDec

    MC_rdec = 1._kp/(1._kp+4._kp*RhoGammaDec/(3._kp*RhoMatterDec))

  end function MC_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the amplitude of the curvature power spectrum
  function MC_Pzeta(Hstar, rdec, dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                    dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)
    implicit none
    real(kp) :: MC_Pzeta
    real(kp), intent(in) :: Hstar, rdec, dlnrho_matter_dmatterfield,dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield

    MC_Pzeta = ((rdec/(3._kp)*dlnrho_matter_dmatterfield + &
        (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dmatterfield)**2+ &
        (rdec/(3._kp)*dlnrho_matter_dGammafield + &
        (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dGammafield)**2)* &
        (Hstar/(2._kp*pi))**2

  end function MC_Pzeta

! Returns the relative contribution of the matter field to the total curvature power spectrum
  function MC_Pzeta_relativecontribution_matter(Hstar, rdec, dlnrho_matter_dmatterfield, &
               dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)
    implicit none
    real(kp) :: MC_Pzeta_relativecontribution_matter
    real(kp), intent(in) :: Hstar, rdec, dlnrho_matter_dmatterfield,dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield

    MC_Pzeta_relativecontribution_matter = ((rdec/(3._kp)*dlnrho_matter_dmatterfield + &
               (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dmatterfield)**2)/ &
               (((rdec/(3._kp)*dlnrho_matter_dmatterfield + &
               (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dmatterfield)**2+ &
               (rdec/(3._kp)*dlnrho_matter_dGammafield + &
               (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dGammafield)**2))

  end function MC_Pzeta_relativecontribution_matter

! Returns the relative contribution of the radiation field to the total curvature power spectrum
  function MC_Pzeta_relativecontribution_radiation(Hstar, rdec, dlnrho_matter_dmatterfield, &
               dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)
    implicit none
    real(kp) :: MC_Pzeta_relativecontribution_radiation
    real(kp), intent(in) :: Hstar, rdec, dlnrho_matter_dmatterfield,dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield

    MC_Pzeta_relativecontribution_radiation = ((rdec/(3._kp)*dlnrho_matter_dGammafield + &
                (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dGammafield)**2)/ &
               (((rdec/(3._kp)*dlnrho_matter_dmatterfield + &
               (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dmatterfield)**2+ &
               (rdec/(3._kp)*dlnrho_matter_dGammafield + &
               (1._kp-rdec)/(4._kp)*dlnrho_Gamma_dGammafield)**2))

  end function MC_Pzeta_relativecontribution_radiation


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!    Local fNL Parameter    !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the fNL parameter as a function of the energy densisties (and their derivatives with respect to the field_* fluctuations) at the last decay time
  function MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                 ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                 dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                 ddlnrhomat_dphirad_dphimat)
    implicit none
    real(kp) :: MC_fnl, Amm, Amr, Arm, Arr, Ammm, Ammr, Amrr, Arrr, Armr, Armm
    real(kp) :: Num, Den
    real(kp), intent(in) :: rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                           ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                           dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                           ddlnrhomat_dphirad_dphimat
    !Axy = d ln(rho_x) / d phi_y
    !Axyz = d^2 ln(rho_x) / d phi_y d phi_z
    Amm=dlnrhomat_dphimat
    Amr=dlnrhomat_dphirad
    Arm=dlnrhorad_dphimat
    Arr=dlnrhorad_dphirad
    Ammm=ddlnrhomat_ddphimat
    Arrr=ddlnrhorad_ddphirad
    Amrr=ddlnrhomat_ddphirad
    Armm=ddlnrhorad_ddphimat
    Ammr=ddlnrhomat_dphirad_dphimat
    Armr=ddlnrhorad_dphirad_dphimat

    MC_fnl = 2._kp*((5._kp*(81._kp*Arr**2*(Armm+2._kp*Arrr)+27._kp*Arr*(8._kp*Amm*Armr+8._kp*Amm**2*Arr &
          +16._kp*Amr**2*Arr+Arr*(4._kp*Ammm+8._kp*Amrr-9._kp*Armm+9._kp*Arr**2-18._kp*Arrr)+ &
           8._kp*Amr*(Armm-3._kp*Arr**2+2._kp*Arrr))*rdec+9._kp*(16._kp*Amm*(2._kp*Amr*Armr+ &
           4._kp*Ammr*Arr-3._kp*Armr*Arr)+16._kp*Amm**2*(2._kp*Armm+2._kp*(6._kp*Amr-5._kp*Arr)* &
           Arr+Arrr)+(4._kp*Amr-3._kp*Arr)*(32._kp*Amr**2*Arr+Arr*(8._kp*Ammm+16._kp*Amrr-9._kp* &
           Armm+24._kp*Arr**2-18._kp*Arrr)+4._kp*Amr*(Armm-14._kp*Arr**2+2._kp*Arrr)))* &
           rdec**2+3._kp*(256._kp*Amm**4+8._kp*Amm*(8._kp*Ammr-3._kp*Armr)*(4._kp*Amr-3._kp*Arr)+ &
           (4._kp*Amr-3._kp*Arr)**2*(4._kp*Ammm+16._kp*Amr**2+8._kp*Amrr-3._kp*Armm-40._kp*Amr* &
           Arr+18._kp*Arr**2-6._kp*Arrr)+8._kp*Amm**2*(16._kp*Ammm+96._kp*Amr**2+8._kp*Amrr- &
           192._kp*Amr*Arr+87._kp*Arr**2-6._kp*(2._kp*Armm+Arrr)))*rdec**3-8._kp*(64._kp* &
           Amm**4+Amr*(4._kp*Amr-3._kp*Arr)**3+6._kp*Amm**2*(4._kp*Amr-3._kp*Arr)*(8._kp*Amr- &
           3._kp*Arr))*rdec**4-(256._kp*Amm**4+48._kp*Amm**2*(4._kp*Amr-3._kp*Arr)**2+(4._kp*Amr- &
           3._kp*Arr)**4)*rdec**5-81._kp*Arm**4*(-1._kp+rdec)**3*rdec*(3._kp+rdec)+216._kp* &
           Amm*Arm**3*(-1._kp+rdec)**2*rdec*(3._kp+rdec)*(-1._kp+2._kp*rdec)+6._kp*Arm*(-1._kp &
           +rdec)*(-9._kp*Armr*(-1._kp+rdec)*(3._kp*Arr*(-1._kp+rdec)-4._kp*Amr*rdec)+2._kp*rdec &
           *(32._kp*Amm**3*rdec*(3._kp+rdec)*(-1._kp+2._kp*rdec)+12._kp*Ammr*(3._kp*Arr*(-1._kp+ &
           rdec)-4._kp*Amr*rdec)+3._kp*Amm*(-6._kp*Arrr+12._kp*Armm*(-1._kp+rdec)-8._kp*(2._kp* &
           Ammm+Amrr)*rdec+16._kp*Amr**2*rdec*(3._kp+rdec)*(-1._kp+2._kp*rdec)-8._kp*Amr*Arr* &
           (3._kp+rdec)*(1._kp+6._kp*(-1._kp+rdec)*rdec)+3._kp*(2._kp*Arrr*rdec+3._kp*Arr**2* &
           (-1._kp+rdec)*(3._kp+rdec)*(-1._kp+2._kp*rdec)))))-9._kp*Arm**2*(-1._kp+rdec)*(18._kp* &
           Armm*(-1._kp+rdec)**2+9._kp*Arrr*(-1._kp+rdec)**2+rdec*(3._kp*(8._kp*Ammm+8._kp*Amr**2+ &
           4._kp*Amrr-36._kp*Amr*Arr+27._kp*Arr**2)+16._kp*Amm**2*(3._kp+rdec)*(1._kp+6._kp*(- &
           1._kp+rdec)*rdec)+rdec*(-24._kp*Ammm-12._kp*Amrr+27._kp*Arr**2*(-5._kp+rdec+rdec**2)+ &
           8._kp*Amr**2*(-17._kp+6._kp*rdec*(2._kp+rdec))-36._kp*Amr*Arr*(-8._kp+rdec*(3._kp+ &
           2._kp*rdec)))))))/(6._kp*(9._kp*Arm**2*(-1._kp+rdec)**2+9._kp*Arr**2*(-1._kp+rdec)**2- &
           24._kp*Amm*Arm*(-1._kp+rdec)*rdec-24._kp*Amr*Arr*(-1._kp+rdec)*rdec+16._kp*(Amm**2+ &
           Amr**2)*rdec**2)**2))

  end function MC_fnl

!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!                        !!
!!        REHEATING       !!
!!                        !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the equation one must solve to obtain bfoldstar
  function find_MC_bfoldstar(V, Vp, Vpp, phiend0, InverseVp_primitive, phi_trajectory, Vparam, &
           msigma, Gammaphi, Gammasigma, sigmaend, lnRrad, Pstar, bfoldstar)
    implicit none
    real(kp) :: find_MC_bfoldstar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, lnRrad, Pstar, bfoldstar
    real(kp) :: phistar, phiend, sigmastar, epsonestar, epsoneend, potend, potstar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
        function phi_trajectory(bfold, Vparam, msigma, sigmaend)
          use infprec, only : kp,transfert
          implicit none
          real(kp), intent(in) :: bfold, msigma, sigmaend
          type(transfert), optional, intent(inout) :: Vparam
          real(kp) :: phi_trajectory
        end function phi_trajectory
    end interface

    phistar = phi_trajectory(bfoldstar, Vparam, msigma, sigmaend)
    phiend = MCphiend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    epsonestar = MC_epsilon_one(V,Vp,Vparam,phistar,sigmastar,msigma)
    epsoneend = MC_epsilon_one(V,Vp,Vparam,phiend,sigmaend,msigma)
    potend = V(phiend,Vparam)+0.5_kp*msigma**2*sigmaend**2
    potstar = V(phistar,Vparam)+0.5_kp*msigma**2*sigmastar**2
    find_MC_bfoldstar = bfoldstar + lnRrad - Nzero - 0.25_kp* &
        log(9._kp/(epsonestar*(3._kp-epsoneend))*potend/potstar)+0.25_kp*log(8._kp*pi**2*Pstar)

  end function find_MC_bfoldstar

! Returns the correlation angle Delta between curvature and isocurvature power spectra
! (see Eqs (35) of astroph/0205253)
  function MC_Delta(V, Vp, Vpp, phiend0, InverseVp_primitive, phi_trajectory, Vparam, &
          msigma, Gammaphi, Gammasigma, sigmaend, bfoldstar)
    implicit none
    real(kp) :: MC_Delta, phistar, sigmastar, potstar, epsOnestar, Pzetastar
    real(kp), intent(in) ::  msigma, Gammaphi, Gammasigma, sigmaend, bfoldstar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
        function phi_trajectory(bfold, Vparam, msigma, sigmaend)
          use infprec, only : kp,transfert
          implicit none
          real(kp), intent(in) :: bfold, msigma, sigmaend
          type(transfert), optional, intent(inout) :: Vparam
          real(kp) :: phi_trajectory
        end function phi_trajectory
    end interface

    phistar = phi_trajectory(bfoldstar, Vparam, msigma, sigmaend)
    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    potstar = V(phistar,Vparam)+0.5_kp*msigma**2*sigmastar**2
    epsonestar = MC_epsilon_one(V,Vp,Vparam,phistar,sigmastar,msigma)
    Pzetastar = potstar/(24._kp*pi**2*epsOnestar)
    MC_Delta = asin(min(sqrt(Pzetastar/powerAmpScalar),1._kp))

  end function MC_Delta

! Returns Delta from already computed intermediary quantites (faciq)
  function MC_Delta_faciq(V, Vp, Vparam, msigma,  phistar, sigmastar)
    implicit none
    real(kp) :: MC_Delta_faciq, potstar, epsOnestar, Pzetastar
    real(kp), intent(in) ::  msigma, phistar, sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
     end interface

    potstar = V(phistar,Vparam)+0.5_kp*msigma**2*sigmastar**2
    epsonestar = MC_epsilon_one(V,Vp,Vparam,phistar,sigmastar,msigma)
    Pzetastar = potstar/(24._kp*pi**2*epsOnestar)
    MC_Delta_faciq = asin(min(sqrt(Pzetastar/powerAmpScalar),1._kp))

  end function MC_Delta_faciq

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC1         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 1
  function MC1_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC1_sigmaend_max

    MC1_sigmaend_max = sqrt(2._kp)*exp(2._kp/9._kp)

  end function MC1_sigmaend_max

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (sigma decay products) at last decay time
  function MC1_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC1_rhogamma_dec, Hend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC1_rhogamma_dec = 1.5_kp*Gammasigma**2*sigmaend**2*(Gammaphi/Gammasigma)**(8._kp/3._kp)* &
                        exp(-4._kp/9._kp+2._kp*msigma**2/(9._kp*Hend**2))

  end function MC1_rhogamma_dec

! Energy density of matter (phi) at last decay time
  function MC1_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC1_rhomatter_dec, RhoGammaDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC1_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC1_rhomatter_dec = 3._kp*Gammaphi**2-RhoGammaDec

  end function MC1_rhomatter_dec

  function MC1_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC1_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC1_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    RhoMatterDec = MC1_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC1_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC1_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_gamma_dec / d sigmastar
  function MC1_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_dlnrho_gamma_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dsigmaenddsigmastar, dHenddsigmastar, dNenddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, &
          msigma, sigmaend)
    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC1_dlnrho_gamma_dec_dsigmastar = 2._kp*dsigmaenddsigmastar/sigmaend+4._kp/Hend* &
                                      (2._kp/3._kp-msigma**2/(9._kp*Hend**2))*dHenddsigmastar+4._kp*dNenddsigmastar

  end function MC1_dlnrho_gamma_dec_dsigmastar

! Returns d lnrho_gamma_dec / d phistar
  function MC1_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
    Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_dlnrho_gamma_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dsigmaenddphistar, dHenddphistar, dNenddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                        sigmaend, phistar, msigma)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, &
                    msigma, sigmaend)
    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, &
                    phistar, msigma)

    MC1_dlnrho_gamma_dec_dphistar = 2._kp*dsigmaenddphistar/sigmaend+4._kp/Hend* &
                        (2._kp/3._kp-msigma**2/(9._kp*Hend**2))*dHenddphistar+4._kp*dNenddphistar

  end function MC1_dlnrho_gamma_dec_dphistar

! Returns d^2 lnrho_gamma_dec / d phistar ^2
  function MC1_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
    Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_gamma_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dsigmaenddphistar, ddsigmaendddphistar, dHenddphistar, ddHendddphistar, ddNendddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface


    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, &
                    msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     sigmaend, phistar, msigma)

    MC1_ddlnrho_gamma_dec_ddphistar = 2._kp/sigmaend*(ddsigmaendddphistar-dsigmaenddphistar**2/ &
                sigmaend)+8._kp/3._kp/Hend*(ddHendddphistar-dHenddphistar**2/Hend+ &
                0.5_kp*msigma**2*dHenddphistar**2/Hend**3- &
                0.5_kp/3._kp*msigma**2*ddHendddphistar/Hend**2)+4._kp*ddNendddphistar

  end function MC1_ddlnrho_gamma_dec_ddphistar

! Returns d^2 lnrho_gamma_dec / d sigmastar ^2
  function MC1_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_gamma_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dsigmaenddsigmastar, ddsigmaendddsigmastar, dHenddsigmastar, &
               ddHendddsigmastar, ddNendddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               phistar, msigma, sigmaend)
    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC1_ddlnrho_gamma_dec_ddsigmastar = 2._kp/sigmaend*(ddsigmaendddsigmastar-dsigmaenddsigmastar**2/&
           sigmaend)+8._kp/3._kp/Hend*(ddHendddsigmastar-dHenddsigmastar**2/Hend+ &
            0.5_kp*msigma**2*dHenddsigmastar**2/Hend**3- &
            0.5_kp/3._kp*msigma**2*ddHendddsigmastar/Hend**2)+4._kp*ddNendddsigmastar

  end function MC1_ddlnrho_gamma_dec_ddsigmastar

! Returns d^2 lnrho_gamma_dec / d phistar d sigmastar
  function MC1_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
    Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_gamma_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) ::  Hend, dsigmaenddsigmastar, dsigmaenddphistar, ddsigmaenddphistardsigmastar, &
                dHenddsigmastar, dHenddphistar, ddHenddphistardsigmastar, ddNenddphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, phistar, &
                    msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC1_ddlnrho_gamma_dec_dphistar_dsigmastar = 2._kp/sigmaend*(ddsigmaenddphistardsigmastar- &
        dsigmaenddsigmastar*dsigmaenddphistar/sigmaend)+8._kp/3._kp/Hend* &
        (ddHenddphistardsigmastar-dHenddsigmastar*dHenddphistar/Hend+ &
        0.5_kp*msigma**2*dHenddsigmastar*dHenddphistar/Hend**3- &
        0.5_kp/3._kp*msigma**2*ddHenddphistardsigmastar/Hend**2)+ &
        4._kp*ddNenddphistardsigmastar

  end function MC1_ddlnrho_gamma_dec_dphistar_dsigmastar

! Returns d lnrho_phi_dec / d sigmastar
  function MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_dlnrho_phi_dec_dsigmastar
    real(kp) :: rhophiend, drhophienddsigmastar, dNenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC1_dlnrho_phi_dec_dsigmastar = drhophienddsigmastar/rhophiend+3._kp*dNenddsigmastar

  end function MC1_dlnrho_phi_dec_dsigmastar

! Returns d lnrho_phi_dec / d phistar
  function MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
    Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_dlnrho_phi_dec_dphistar
    real(kp) :: rhophiend, drhophienddphistar, dNenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, &
                    phistar, msigma)

    MC1_dlnrho_phi_dec_dphistar = drhophienddphistar/rhophiend+3._kp*dNenddphistar

  end function MC1_dlnrho_phi_dec_dphistar

! Returns d^2 lnrho_phi_dec / d phistar ^2
  function MC1_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
        Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_phi_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: rhophiend, drhophienddphistar, ddrhophiendddphistar, ddNendddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                      sigmaend, phistar, msigma)

    MC1_ddlnrho_phi_dec_ddphistar = ddrhophiendddphistar/rhophiend-drhophienddphistar**2/ &
          rhophiend**2+3._kp*ddNendddphistar

  end function MC1_ddlnrho_phi_dec_ddphistar

! Returns d^2 lnrho_phi_dec / d sigmastar ^2
  function MC1_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
           Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_phi_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: rhophiend, drhophienddsigmastar, ddrhophiendddsigmastar, ddNendddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        sigmaend, phistar, msigma)

    MC1_ddlnrho_phi_dec_ddsigmastar = ddrhophiendddsigmastar/rhophiend-drhophienddsigmastar**2/ &
        rhophiend**2+3._kp*ddNendddsigmastar

  end function MC1_ddlnrho_phi_dec_ddsigmastar

! Returns d^2 lnrho_phi_dec / d phistar d sigmastar
  function MC1_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
         msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_ddlnrho_phi_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: rhophiend, drhophienddsigmastar,  drhophienddphistar, ddrhophienddphistardsigmastar, &
               ddNenddphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC1_ddlnrho_phi_dec_dphistar_dsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
        drhophienddsigmastar*drhophienddphistar/rhophiend**2+3._kp*ddNenddphistardsigmastar

  end function MC1_ddlnrho_phi_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC1_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC1_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC1_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC1_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC1_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC1_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC1_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC1_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC1_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC1_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC1_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_radiation(Hstar, &
                rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC1_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC1_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC1_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC1_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC1_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC1_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC1_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC1_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC1_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC1_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC1_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC1_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC1_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC1_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC1_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC1_lnRrad, rhoend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    rhoend = 3._kp*MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)**2
    MC1_lnRrad = 1._kp/12._kp*log(3._kp*Gammaphi**2/rhoend)

  end function MC1_lnRrad

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC2         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 2
  function MC2_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC2_sigmaend_max

    if (Gammaphi/Gammasigma < 10._kp**(-15)) then !Uses an approximate expression for the Lambert function
      MC2_sigmaend_max = sqrt(2._kp-4._kp/3._kp*log(9._kp/8._kp)-2._kp/3._kp* &
                         log(Gammaphi/Gammasigma))
    else
      MC2_sigmaend_max = sqrt(-4._kp/3._kp*lambert(-9._kp/8._kp*exp(-1.5_kp)* &
                          sqrt(Gammaphi/Gammasigma),-1))
    end if

  end function MC2_sigmaend_max

!returns the minimum allowed value for sigmaend for case 2
  function MC2_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC2_sigmaend_min

    MC2_sigmaend_min = 1.6914166820639671249587974497202_kp ! = sqrt(-4/3 W_{-1} (-9/8 Exp(-3/2) ))

  end function MC2_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! First equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of first equality between energy densities of phi and sigma
  function MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_Heq1
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    real(kp) :: Hend, rhophiend, xmini, xmaxi, x ! x is (Heq/msigma)^2
    type(transfert) :: MCData
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MCData%real1 = msigma
    MCData%real2 = Gammaphi
    MCData%real3 = Gammasigma
    MCData%real4 = sigmaend

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MCData%real5 = Hend
    MCData%real6 = rhophiend

    xmini = sqrt(2._kp)/9._kp ! where find_MC2_Heq1 is maximal
    xmaxi = (Hend/msigma)**2*5._kp ! 5 is a safety number

    x = zbrent(find_MC2_Heq1,xmini,xmaxi,tolFind,MCData)

    MC2_Heq1 = msigma*sqrt(x)

  end function MC2_Heq1

! Returns the equation one must solve to obtain Heq1
  function find_MC2_Heq1(x,MCData)
    implicit none
    real(kp) :: find_MC2_Heq1
    real(kp), intent(in) :: x ! x is (Heq/msigma)^2
    type(transfert), optional, intent(inout) :: MCData
    real(kp) :: mphi, msigma, Gammaphi, Gammasigma, sigmaend, Hend, rhophiend

    msigma = MCData%real1
    Gammaphi = MCData%real2
    Gammasigma = MCData%real3
    sigmaend = MCData%real4
    Hend = MCData%real5
    rhophiend = MCData%real6

    find_MC2_Heq1 = 0.5_kp*sigmaend**2*Hend**2/rhophiend*exp(-2._kp/(9._kp*x)+ &
                     2._kp*msigma**2/(9._kp*Hend**2))*(1._kp+1._kp/(9._kp*x))/x-1._kp

  end function find_MC2_Heq1

! Returns sigma at the first equivalence time, as a function of Heq1
  function MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    real(kp) :: MC2_sigmaeq1, Hend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, Heq1
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC2_sigmaeq1 = sigmaend*exp(msigma**2/(9._kp*Heq1**2)*((Heq1/Hend)**2-1._kp))

  end function MC2_sigmaeq1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Second equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of second equality between energy densities of radiation and phi
  function MC2_Heq2(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_Heq2
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    real(kp) :: rhophiend, Hend, Heq1, sigmaeq1
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)

    MC2_Heq2 = 16._kp/9._kp*Gammasigma*rhophiend**2/msigma**4* &
                (Heq1/Hend)**4*exp(3._kp-1.5_kp*sigmaeq1**2)


  end function MC2_Heq2

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (sigma decay products) at last decay time
  function MC2_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_rhogamma_dec, Heq2
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Heq2 = MC2_Heq2(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC2_rhogamma_dec = 3._kp*Gammaphi**(8._kp/3._kp)/Heq2**(2._kp/3._kp)

  end function MC2_rhogamma_dec

! Energy density of matter (phi) at last decay time
  function MC2_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_rhomatter_dec, RhoGammaDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC2_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                  Gammasigma, sigmaend)
    MC2_rhomatter_dec = 3._kp*Gammaphi**2-RhoGammaDec

  end function MC2_rhomatter_dec

  function MC2_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC2_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                   Gammasigma, sigmaend)
    RhoMatterDec = MC2_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                   Gammasigma, sigmaend)
    MC2_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC2_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!   d Heq1 / d sigma* and phi*   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnHeq1 / d phistar
  function MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnHeq1_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    real(kp) :: dsigmaenddphistar, drhophienddphistar, dlnrhophienddphistar, &
                dHenddphistar, dlnHenddphistar, rhophiend, Hend, Heq1
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                         Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnrhophienddphistar = drhophienddphistar/rhophiend
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_dlnHeq1_dphistar = (dsigmaenddphistar/sigmaend-0.5_kp*dlnrhophienddphistar+(1._kp- &
                            2._kp*msigma**2/(9._kp*Hend**2))*dlnHenddphistar)/(1._kp+msigma**2/ &
                            (msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp*Heq1**2))

  end function MC2_dlnHeq1_dphistar

! Returns d^2 lnHeq1 / d phistar ^2
  function MC2_ddlnHeq1_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnHeq1_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, drhophienddphistar, dlnrhophienddphistar, &
                dHenddphistar, dlnHenddphistar, rhophiend, Hend, Heq1, ddsigmaendddphistar, &
                ddrhophiendddphistar, ddlnrhophiendddphistar, ddHendddphistar, &
                ddlnHendddphistar, dlnHeq1dphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dlnrhophienddphistar = drhophienddphistar/rhophiend
     ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, &
                            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddphistar = ddrhophiendddphistar/rhophiend-(drhophienddphistar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend-(dHenddphistar/Hend)**2
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnHeq1dphistar = MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC2_ddlnHeq1_ddphistar = (ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/ &
        sigmaend**2-0.5_kp*ddlnrhophiendddphistar+(1._kp-2._kp*msigma**2/(9._kp*Hend**2))* &
        ddlnHendddphistar+4._kp*msigma**2/(9._kp*Hend**2)*dlnHenddphistar**2)/ &
        (1._kp+msigma**2/(msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp* &
        Heq1**2))-2._kp*dlnHeq1dphistar*(2._kp*msigma**2/(9._kp*Heq1**2)-9._kp* &
        msigma**2*Heq1**2/(msigma**2+9._kp*Heq1**2)**2)*(dsigmaenddphistar/ &
        sigmaend-0.5_kp*dlnrhophienddphistar+(1._kp- 2._kp*msigma**2/ &
        (9._kp*Hend**2))*dlnHenddphistar)/(1._kp+msigma**2/ &
        (msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp*Heq1**2))**2

  end function MC2_ddlnHeq1_ddphistar


! Returns d lnHeq1 / d sigmastar
  function MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
             msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnHeq1_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, drhophienddsigmastar, dlnrhophienddsigmastar, &
                dHenddsigmastar, dlnHenddsigmastar, rhophiend, Hend, Heq1
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_dlnHeq1_dsigmastar = (dsigmaenddsigmastar/sigmaend-0.5_kp*dlnrhophienddsigmastar+(1._kp- &
                2._kp*msigma**2/(9._kp*Hend**2))*dlnHenddsigmastar)/(1._kp+msigma**2/ &
                (msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp*Heq1**2))

  end function MC2_dlnHeq1_dsigmastar

! Returns d^2 lnHeq1 / d sigmastar ^2
  function MC2_ddlnHeq1_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnHeq1_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, drhophienddsigmastar, dlnrhophienddsigmastar, &
                dHenddsigmastar, dlnHenddsigmastar, rhophiend, Hend, Heq1, &
                ddsigmaendddsigmastar, ddrhophiendddsigmastar, ddlnrhophiendddsigmastar, &
                ddHendddsigmastar, ddlnHendddsigmastar, dlnHeq1dsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface


    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddsigmastar = ddrhophiendddsigmastar/rhophiend-(drhophienddsigmastar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend-(dHenddsigmastar/Hend)**2
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnHeq1dsigmastar = MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    MC2_ddlnHeq1_ddsigmastar = (ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/ &
            sigmaend**2-0.5_kp*ddlnrhophiendddsigmastar+(1._kp-2._kp*msigma**2/(9._kp* &
            Hend**2))*ddlnHendddsigmastar+4._kp*msigma**2/(9._kp*Hend**2)* &
            dlnHenddsigmastar**2)/ &
            (1._kp+msigma**2/(msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp* &
            Heq1**2))-2._kp*dlnHeq1dsigmastar*(2._kp*msigma**2/(9._kp*Heq1**2)-9._kp* &
            msigma**2*Heq1**2/(msigma**2+9._kp*Heq1**2)**2)*(dsigmaenddsigmastar/&
            sigmaend-0.5_kp*dlnrhophienddsigmastar+(1._kp- 2._kp*msigma**2/ &
            (9._kp*Hend**2))*dlnHenddsigmastar)/(1._kp+msigma**2/ &
            (msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp*Heq1**2))**2

  end function MC2_ddlnHeq1_ddsigmastar

! Returns d^2 lnHeq1 / d phistar d sigmastar
  function MC2_ddlnHeq1_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnHeq1_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: rhophiend, dsigmaenddsigmastar, dsigmaenddphistar, drhophienddsigmastar, &
                dlnrhophienddsigmastar, dHenddsigmastar, dHenddphistar, &
                dlnHenddsigmastar, dlnHenddphistar, Hend, Heq1, &
                ddsigmaenddphistardsigmastar, ddrhophienddphistardsigmastar, &
                ddlnrhophienddphistardsigmastar, ddHenddphistardsigmastar, &
                ddlnHenddphistardsigmastar, dlnHeq1dphistar, drhophienddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHenddphistar = dHenddphistar/Hend
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophienddphistardsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
                                      drhophienddsigmastar*drhophienddphistar/rhophiend**2
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                                 dHenddsigmastar*dHenddphistar/Hend**2
    dlnHeq1dphistar = MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC2_ddlnHeq1_dphistar_dsigmastar = (ddsigmaenddphistardsigmastar/sigmaend- &
        dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2-0.5_kp* &
        ddlnrhophienddphistardsigmastar+&
        (1._kp-2._kp*msigma**2/(9._kp*Hend**2 ))*ddlnHenddphistardsigmastar+ &
        4._kp*msigma**2/(9._kp*Hend**2)*dlnHenddsigmastar*dlnHenddphistar)/ &
        (1._kp+msigma**2/(msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp* &
        Heq1**2))-2._kp*dlnHeq1dphistar*(2._kp*msigma**2/(9._kp*Heq1**2)-9._kp* &
        msigma**2*Heq1**2/(msigma**2+9._kp*Heq1**2)**2)*(dsigmaenddsigmastar/&
        sigmaend-0.5_kp*dlnrhophienddsigmastar+(1._kp- 2._kp*msigma**2/ &
        (9._kp*Hend**2))*dlnHenddsigmastar)/(1._kp+msigma**2/ &
        (msigma**2+9._kp*Heq1**2)-2._kp*msigma**2/(9._kp*Heq1**2))**2

  end function MC2_ddlnHeq1_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  d sigmaeq1 / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d ln sigmaeq1 / d phistar
  function MC2_dlnsigmaeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnsigmaeq1_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, dHenddphistar, dlnHenddphistar, Hend, Heq1, dlnHeq1dphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHeq1dphistar =  MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_dlnsigmaeq1_dphistar = dsigmaenddphistar/sigmaend-2._kp*msigma**2/(9._kp*Hend**2)* &
                                dlnHenddphistar+2._kp*msigma**2/(9._kp*Heq1**2)*dlnHeq1dphistar

  end function MC2_dlnsigmaeq1_dphistar

! Returns d^2 ln sigmaeq1 / d phistar ^2
  function MC2_ddlnsigmaeq1_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnsigmaeq1_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, ddsigmaendddphistar, ddlnsigmaendddphistar, dHenddphistar, &
                ddHendddphistar, dlnHenddphistar, ddlnHendddphistar, Hend, Heq1, &
                dlnHeq1dphistar, ddlnHeq1ddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddphistar = ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    dlnHeq1dphistar =  MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeq1ddphistar = MC2_ddlnHeq1_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_ddlnsigmaeq1_ddphistar = ddlnsigmaendddphistar-2._kp*msigma**2/(9._kp*Hend**2)* &
                                 (ddlnHendddphistar-2._kp*dlnHenddphistar**2)+ &
                                  2._kp*msigma**2/(9._kp*Heq1**2)*(ddlnHeq1ddphistar- &
                                  2._kp*dlnHeq1dphistar**2)

  end function MC2_ddlnsigmaeq1_ddphistar

! Returns d ln sigmaeq1 / d sigmastar
  function MC2_dlnsigmaeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnsigmaeq1_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, dHenddsigmastar, dlnHenddsigmastar, Hend, Heq1, &
                dlnHeq1dsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHeq1dsigmastar = MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_dlnsigmaeq1_dsigmastar = dsigmaenddsigmastar/sigmaend-2._kp*msigma**2/(9._kp*Hend**2)* &
                                dlnHenddsigmastar+2._kp*msigma**2/(9._kp*Heq1**2)*dlnHeq1dsigmastar

  end function MC2_dlnsigmaeq1_dsigmastar

! Returns d^2 ln sigmaeq1 / d sigmastar ^2
  function MC2_ddlnsigmaeq1_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnsigmaeq1_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, ddsigmaendddsigmastar, ddlnsigmaendddsigmastar, &
                dHenddsigmastar, ddHendddsigmastar, dlnHenddsigmastar, ddlnHendddsigmastar, &
                Hend, Heq1, dlnHeq1dsigmastar, ddlnHeq1ddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddsigmastar = ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    dlnHeq1dsigmastar = MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeq1ddsigmastar = MC2_ddlnHeq1_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_ddlnsigmaeq1_ddsigmastar = ddlnsigmaendddsigmastar-2._kp*msigma**2/(9._kp*Hend**2)* &
                                 (ddlnHendddsigmastar-2._kp*dlnHenddsigmastar**2)+ &
                                  2._kp*msigma**2/(9._kp*Heq1**2)*(ddlnHeq1ddsigmastar- &
                                  2._kp*dlnHeq1dsigmastar**2)

  end function MC2_ddlnsigmaeq1_ddsigmastar

! Returns d^2 ln sigmaeq1 / d phistar d sigmastar
  function MC2_ddlnsigmaeq1_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnsigmaeq1_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, dsigmaenddphistar, ddsigmaenddphistardsigmastar, &
                ddlnsigmaenddphistardsigmastar, dHenddsigmastar, dHenddphistar, &
                ddHenddphistardsigmastar, dlnHenddsigmastar, dlnHenddphistar, &
                ddlnHenddphistardsigmastar, Hend, Heq1, dlnHeq1dsigmastar, dlnHeq1dphistar, &
                ddlnHeq1dphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                          sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnsigmaenddphistardsigmastar = ddsigmaenddphistardsigmastar/sigmaend- &
          dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHenddphistar = dHenddphistar/Hend
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend - &
               dHenddsigmastar*dHenddphistar/Hend**2
    dlnHeq1dsigmastar = MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnHeq1dphistar =  MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeq1dphistardsigmastar = MC2_ddlnHeq1_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
              InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC2_ddlnsigmaeq1_dphistar_dsigmastar = ddlnsigmaenddphistardsigmastar-2._kp*msigma**2/ &
        (9._kp*Hend**2)*(ddlnHenddphistardsigmastar-2._kp* &
        dlnHenddsigmastar*dlnHenddphistar)+2._kp*msigma**2/ &
        (9._kp*Heq1**2)*(ddlnHeq1dphistardsigmastar- &
        2._kp*dlnHeq1dsigmastar*dlnHeq1dphistar)

  end function MC2_ddlnsigmaeq1_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_gamma_dec / d sigmastar
  function MC2_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnrho_gamma_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddsigmastar, dlnHeq1dsigmastar, Heq1, sigmaeq1, dlnsigmaeq1dsigmastar, &
               dNenddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dlnHeq1dsigmastar = MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    dlnsigmaeq1dsigmastar = MC2_dlnsigmaeq1_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC2_dlnrho_gamma_dec_dsigmastar = 8._kp/3._kp*(dHenddsigmastar/Hend-dlnHeq1dsigmastar)+ &
                2._kp*sigmaeq1**2*dlnsigmaeq1dsigmastar+4._kp*dNenddsigmastar

  end function MC2_dlnrho_gamma_dec_dsigmastar

! Returns d^2 lnrho_gamma_dec / d sigmastar ^2
  function MC2_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_gamma_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddsigmastar, ddHendddsigmastar, ddlnHendddsigmastar, &
                ddlnHeq1ddsigmastar, Heq1, sigmaeq1, dlnsigmaeq1dsigmastar, &
                ddlnsigmaeq1ddsigmastar, ddNendddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    ddlnHeq1ddsigmastar = MC2_ddlnHeq1_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    dlnsigmaeq1dsigmastar = MC2_dlnsigmaeq1_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeq1ddsigmastar = MC2_ddlnsigmaeq1_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        sigmaend, phistar, msigma)

    MC2_ddlnrho_gamma_dec_ddsigmastar = 8._kp/3._kp*(ddlnHendddsigmastar-ddlnHeq1ddsigmastar)+ &
                    2._kp*sigmaeq1**2*ddlnsigmaeq1ddsigmastar+4._kp*sigmaeq1**2*&
                    dlnsigmaeq1dsigmastar**2+4._kp*ddNendddsigmastar

  end function MC2_ddlnrho_gamma_dec_ddsigmastar

! Returns d lnrho_gamma_dec / d phistar
  function MC2_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnrho_gamma_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, dlnHeq1dphistar, Heq1, sigmaeq1, dlnsigmaeq1dphistar, &
               dNenddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHeq1dphistar = MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    dlnsigmaeq1dphistar = MC2_dlnsigmaeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                sigmaend, phistar, msigma)

    MC2_dlnrho_gamma_dec_dphistar = 8._kp/3._kp*(dHenddphistar/Hend-dlnHeq1dphistar)+2._kp* &
                                       sigmaeq1**2*dlnsigmaeq1dphistar+4._kp*dNenddphistar

  end function MC2_dlnrho_gamma_dec_dphistar

! Returns d^2 lnrho_gamma_dec / d phistar ^2
  function MC2_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_gamma_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, ddHendddphistar, ddlnHendddphistar, ddlnHeq1ddphistar, &
                Heq1, sigmaeq1, dlnsigmaeq1dphistar, ddlnsigmaeq1ddphistar, ddNendddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    ddlnHeq1ddphistar = MC2_ddlnHeq1_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    dlnsigmaeq1dphistar = MC2_dlnsigmaeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeq1ddphistar = MC2_ddlnsigmaeq1_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                      sigmaend, phistar, msigma)

    MC2_ddlnrho_gamma_dec_ddphistar = 8._kp/3._kp*(ddlnHendddphistar-ddlnHeq1ddphistar)+ &
                    2._kp*sigmaeq1**2*ddlnsigmaeq1ddphistar+4._kp*sigmaeq1**2*&
                    dlnsigmaeq1dphistar**2+4._kp*ddNendddphistar

  end function MC2_ddlnrho_gamma_dec_ddphistar

! Returns d^2 lnrho_gamma_dec / d phistar d sigmastar
  function MC2_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_gamma_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, dHenddsigmastar, ddHenddphistardsigmastar, &
                ddlnHenddphistardsigmastar, ddlnHeq1dphistardsigmastar, Heq1, &
                sigmaeq1, dlnsigmaeq1dphistar, dlnsigmaeq1dsigmastar, &
                ddlnsigmaeq1dphistardsigmastar, ddNenddphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    ddlnHeq1dphistardsigmastar = MC2_ddlnHeq1_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
              InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)
    dlnsigmaeq1dphistar = MC2_dlnsigmaeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnsigmaeq1dsigmastar = MC2_dlnsigmaeq1_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeq1dphistardsigmastar = MC2_ddlnsigmaeq1_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC2_ddlnrho_gamma_dec_dphistar_dsigmastar = 8._kp/3._kp*(ddlnHenddphistardsigmastar- &
                    ddlnHeq1dphistardsigmastar)+2._kp*sigmaeq1**2* &
                    ddlnsigmaeq1dphistardsigmastar+4._kp*sigmaeq1**2*&
                    dlnsigmaeq1dphistar*dlnsigmaeq1dsigmastar+4._kp*&
                    ddNenddphistardsigmastar

  end function MC2_ddlnrho_gamma_dec_dphistar_dsigmastar

! Returns d lnrho_phi_dec / d sigmastar
function MC2_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnrho_phi_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC2_dlnrho_phi_dec_dsigmastar = MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC2_dlnrho_phi_dec_dsigmastar

! Returns d lnrho_phi_dec / d phistar
function MC2_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_dlnrho_phi_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC2_dlnrho_phi_dec_dphistar = MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC2_dlnrho_phi_dec_dphistar

! Returns d^2 lnrho_phi_dec / d phistar ^2
function MC2_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_phi_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC2_ddlnrho_phi_dec_ddphistar = MC1_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC2_ddlnrho_phi_dec_ddphistar

! Returns d^2 lnrho_phi_dec / d sigmastar ^2
function MC2_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_phi_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC2_ddlnrho_phi_dec_ddsigmastar = MC1_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC2_ddlnrho_phi_dec_ddsigmastar

! Returns d^2 lnrho_phi_dec / d phistar d sigmastar
function MC2_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_ddlnrho_phi_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC2_ddlnrho_phi_dec_dphistar_dsigmastar = MC1_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, &
                Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
                sigmaend, phistar)

  end function MC2_ddlnrho_phi_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC2_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC2_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC2_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC2_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC2_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC2_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC2_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC2_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC2_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
           Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC2_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC2_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC2_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC2_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC2_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC2_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_radiation(Hstar, &
                           rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC2_Pzeta_relativecontribution_sigma



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC2_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC2_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC2_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC2_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC2_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC2_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC2_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC2_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC2_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC2_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC2_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC2_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC2_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC2_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC2_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC2_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC2_lnRrad, Hend, Heq1, Heq2, sigmaeq1
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq1 = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    Heq2 = MC2_Heq2(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq1 = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq1)

    MC2_lnRrad = 1._kp/6._kp*log(Heq1/Hend)+0.5_kp-0.25_kp*sigmaeq1**2+ &
                  0.25_kp*log(2._kp*Heq1**2/msigma**2)+ &
                  1._kp/12._kp*log(2._kp*Gammasigma**2/msigma**2)+ &
                  1._kp/6._kp*log(Gammaphi/Heq2)

  end function MC2_lnRrad

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC3         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 3
  function MC3_sigmaend_max(V,phiend0,Vparam,msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC3_sigmaend_max, Vend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Vend = V(phiend0(Vparam),Vparam)

    MC3_sigmaend_max = sqrt(2._kp*Vend)/msigma

  end function MC3_sigmaend_max

!returns the minimum allowed value for sigmaend for case 2
  function MC3_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC3_sigmaend_min

    MC3_sigmaend_min = MC2_sigmaend_max(msigma,Gammaphi,Gammasigma)

  end function MC3_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!
!! equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of equality between energy densities of phi and sigma
  function MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    type(transfert) :: MCData
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC3_Heq = MC2_Heq1(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC3_Heq

! Returns sigma at the equivalence time, as a function of Heq
  function MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    real(kp) :: MC3_sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, Heq
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC3_sigmaeq = MC2_sigmaeq1(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, &
                  sigmaend, Heq)

  end function MC3_sigmaeq


!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (sigma decay products) at last decay time
  function MC3_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_rhogamma_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC3_rhogamma_dec = 3._kp*Gammaphi**2

  end function MC3_rhogamma_dec

! Energy density of matter (phi) at last decay time
  function MC3_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_rhomatter_dec, rhophiend, Heq, Hend, sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    MC3_rhomatter_dec = rhophiend*Heq**2/Hend**2*exp(1.5_kp-0.75_kp*sigmaeq**2)* &
                         2._kp*(Gammasigma/msigma)**2*(Gammaphi/Gammasigma)**(1.5)

  end function MC3_rhomatter_dec

  function MC3_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC3_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    RhoMatterDec = MC3_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC3_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC3_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d H_eq / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnHeq / d phistar
  function MC3_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnHeq_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnHeq_dphistar = MC2_dlnHeq1_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
              Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_dlnHeq_dphistar

! Returns d^2 lnHeq / d phistar ^2
  function MC3_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnHeq_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnHeq_ddphistar = MC2_ddlnHeq1_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
         Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnHeq_ddphistar


! Returns d lnHeq / d sigmastar
  function MC3_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
             msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnHeq_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnHeq_dsigmastar =  MC2_dlnHeq1_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)


  end function MC3_dlnHeq_dsigmastar

! Returns d^2 lnHeq / d sigmastar ^2
  function MC3_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnHeq_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnHeq_ddsigmastar = MC2_ddlnHeq1_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnHeq_ddsigmastar

! Returns d^2 lnHeq / d phistar d sigmastar
  function MC3_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnHeq_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnHeq_dphistar_dsigmastar =  MC2_ddlnHeq1_dphistar_dsigmastar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnHeq_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  d sigmaeq / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d ln sigmaeq / d phistar
  function MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnsigmaeq_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnsigmaeq_dphistar = MC2_dlnsigmaeq1_dphistar(V, Vp, Vpp, phiend0, &
       InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_dlnsigmaeq_dphistar

! Returns d^2 ln sigmaeq / d phistar ^2
  function MC3_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnsigmaeq_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnsigmaeq_ddphistar = MC2_ddlnsigmaeq1_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnsigmaeq_ddphistar

! Returns d ln sigmaeq / d sigmastar
  function MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnsigmaeq_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnsigmaeq_dsigmastar = MC2_dlnsigmaeq1_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_dlnsigmaeq_dsigmastar

! Returns d^2 ln sigmaeq / d sigmastar ^2
  function MC3_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnsigmaeq_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnsigmaeq_ddsigmastar = MC2_ddlnsigmaeq1_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnsigmaeq_ddsigmastar

! Returns d^2 ln sigmaeq / d phistar d sigmastar
  function MC3_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnsigmaeq_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnsigmaeq_dphistar_dsigmastar = MC2_ddlnsigmaeq1_dphistar_dsigmastar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnsigmaeq_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_gamma_dec / d sigmastar
  function MC3_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnrho_gamma_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddsigmastar, dlnHeqdsigmastar, Heq, sigmaeq, dlnsigmaeqdsigmastar, &
               dNenddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dlnHeqdsigmastar = MC3_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC3_dlnrho_gamma_dec_dsigmastar = 8._kp/3._kp*(dHenddsigmastar/Hend-dlnHeqdsigmastar)+ &
                2._kp*sigmaeq**2*dlnsigmaeqdsigmastar+4._kp*dNenddsigmastar

  end function MC3_dlnrho_gamma_dec_dsigmastar

! Returns d^2 lnrho_gamma_dec / d sigmastar ^2
  function MC3_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_gamma_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddsigmastar, ddHendddsigmastar, ddlnHendddsigmastar, &
                ddlnHeqddsigmastar, Heq, sigmaeq, dlnsigmaeqdsigmastar, &
                ddlnsigmaeqddsigmastar, ddNendddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    ddlnHeqddsigmastar = MC3_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddsigmastar = MC3_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        sigmaend, phistar, msigma)

    MC3_ddlnrho_gamma_dec_ddsigmastar = 8._kp/3._kp*(ddlnHendddsigmastar-ddlnHeqddsigmastar)+ &
                    2._kp*sigmaeq**2*ddlnsigmaeqddsigmastar+4._kp*sigmaeq**2*&
                    dlnsigmaeqdsigmastar**2+4._kp*ddNendddsigmastar

  end function MC3_ddlnrho_gamma_dec_ddsigmastar

! Returns d lnrho_gamma_dec / d phistar
  function MC3_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnrho_gamma_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, dlnHeqdphistar, Heq, sigmaeq, dlnsigmaeqdphistar, &
               dNenddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHeqdphistar = MC3_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                sigmaend, phistar, msigma)

    MC3_dlnrho_gamma_dec_dphistar = 8._kp/3._kp*(dHenddphistar/Hend-dlnHeqdphistar)+2._kp* &
                                       sigmaeq**2*dlnsigmaeqdphistar+4._kp*dNenddphistar

  end function MC3_dlnrho_gamma_dec_dphistar

! Returns d^2 lnrho_gamma_dec / d phistar ^2
  function MC3_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive,Vparam, &
                msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_gamma_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, ddHendddphistar, ddlnHendddphistar, ddlnHeqddphistar, &
                Heq, sigmaeq, dlnsigmaeqdphistar, ddlnsigmaeqddphistar, ddNendddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    ddlnHeqddphistar = MC3_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddphistar = MC3_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                      sigmaend, phistar, msigma)

    MC3_ddlnrho_gamma_dec_ddphistar = 8._kp/3._kp*(ddlnHendddphistar-ddlnHeqddphistar)+ &
                    2._kp*sigmaeq**2*ddlnsigmaeqddphistar+4._kp*sigmaeq**2*&
                    dlnsigmaeqdphistar**2+4._kp*ddNendddphistar

  end function MC3_ddlnrho_gamma_dec_ddphistar

! Returns d^2 lnrho_gamma_dec / d phistar d sigmastar
  function MC3_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_gamma_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hend, dHenddphistar, dHenddsigmastar, ddHenddphistardsigmastar, &
                ddlnHenddphistardsigmastar, ddlnHeqdphistardsigmastar, Heq, &
                sigmaeq, dlnsigmaeqdphistar, dlnsigmaeqdsigmastar, &
                ddlnsigmaeqdphistardsigmastar, ddNenddphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    ddlnHeqdphistardsigmastar = MC3_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
              InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqdphistardsigmastar = MC3_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC3_ddlnrho_gamma_dec_dphistar_dsigmastar = 8._kp/3._kp*(ddlnHenddphistardsigmastar- &
                    ddlnHeqdphistardsigmastar)+2._kp*sigmaeq**2* &
                    ddlnsigmaeqdphistardsigmastar+4._kp*sigmaeq**2*&
                    dlnsigmaeqdphistar*dlnsigmaeqdsigmastar+4._kp*&
                    ddNenddphistardsigmastar

  end function MC3_ddlnrho_gamma_dec_dphistar_dsigmastar

! Returns d lnrho_phi_dec / d sigmastar
function MC3_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnrho_phi_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnrho_phi_dec_dsigmastar = MC1_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_dlnrho_phi_dec_dsigmastar

! Returns d lnrho_phi_dec / d phistar
function MC3_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_dlnrho_phi_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_dlnrho_phi_dec_dphistar = MC1_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_dlnrho_phi_dec_dphistar

! Returns d^2 lnrho_phi_dec / d phistar ^2
function MC3_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_phi_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnrho_phi_dec_ddphistar = MC1_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnrho_phi_dec_ddphistar

! Returns d^2 lnrho_phi_dec / d sigmastar ^2
function MC3_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_phi_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnrho_phi_dec_ddsigmastar = MC1_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC3_ddlnrho_phi_dec_ddsigmastar

! Returns d^2 lnrho_phi_dec / d phistar d sigmastar
function MC3_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_ddlnrho_phi_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC3_ddlnrho_phi_dec_dphistar_dsigmastar = MC1_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, &
                Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
                sigmaend, phistar)

  end function MC3_ddlnrho_phi_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC3_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC3_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC3_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC3_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC3_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC3_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC3_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC3_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC3_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC3_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC3_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC3_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC3_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC3_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC3_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_radiation(Hstar, &
                rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC3_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC3_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC3_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC3_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC3_dlnrho_phi_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC3_dlnrho_gamma_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC3_dlnrho_phi_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC3_dlnrho_gamma_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC3_ddlnrho_gamma_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC3_ddlnrho_gamma_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC3_ddlnrho_gamma_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC3_ddlnrho_phi_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC3_ddlnrho_phi_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC3_ddlnrho_phi_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC3_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC3_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC3_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_lnRrad, Hend, Heq, sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)

    MC3_lnRrad = 1._kp/12._kp*log(2._kp*Gammasigma**2*Heq**2/(Hend**2*msigma**2))+0.5_kp- &
                   0.25_kp*sigmaeq**2+0.25_kp*log(2._kp*Heq**2/msigma**2)

  end function MC3_lnRrad

! Returns the minimum value of M so that the CMB pivot scale crosses the Hubble radius during the first inflationary period. WARNING: This function must be called with M=1 in the Vparam argument.
  function MC3_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC3_Mmin_reh, Vend0, phiend0val
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam) ! This is Vend0/M^4, this function must be called with M=1 in the Vparam argument

    MC3_Mmin_reh = exp(-log(sigmaend)+0.25_kp*log(Vend0/Gammasigma)-0.75_kp+3._kp/ &
            8._kp*sigmaend**2-11._kp/8._kp*log(2._kp)+2._kp*log(3._kp)- &
            3._kp/8._kp*log(Vend0)+1.5_kp*Nzero)

  end function MC3_Mmin_reh

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC4         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 4
  function MC4_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC4_sigmaend_max

    MC4_sigmaend_max = sqrt(2._kp)*exp(2._kp/9._kp)*(Gammasigma/Gammaphi)**(0.25_kp)

  end function MC4_sigmaend_max

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC4_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC4_rhogamma_dec, Hend, rhophiend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC4_rhogamma_dec = rhophiend*(Gammasigma/Hend)**2

  end function MC4_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC4_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC4_rhomatter_dec, Hend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC4_rhomatter_dec = 1.5_kp*Gammaphi**2*(Gammasigma/Gammaphi)**(1.5_kp)* &
                    sigmaend**2*exp(-4._kp/9._kp+2._kp/9._kp*(msigma/Hend)**2)

  end function MC4_rhomatter_dec

  function MC4_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC4_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC4_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    RhoMatterDec = MC4_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC4_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC4_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC4_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_dlnrho_rad_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dNenddsigmastar, drhophienddsigmastar, rhophiend, dlnrhophienddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend

    MC4_dlnrho_rad_dec_dsigmastar = 4._kp*dNenddsigmastar+4._kp/3._kp*dlnrhophienddsigmastar

  end function MC4_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC4_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_dlnrho_rad_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dNenddphistar, drhophienddphistar, rhophiend, dlnrhophienddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnrhophienddphistar = drhophienddphistar/rhophiend

    MC4_dlnrho_rad_dec_dphistar = 4._kp*dNenddphistar+4._kp/3._kp*dlnrhophienddphistar

  end function MC4_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC4_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_rad_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: ddNendddsigmastar, ddrhophiendddsigmastar, drhophienddsigmastar, rhophiend, &
                ddlnrhophiendddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    ddlnrhophiendddsigmastar = ddrhophiendddsigmastar/rhophiend-(drhophienddsigmastar/rhophiend)**2

    MC4_ddlnrho_rad_dec_ddsigmastar = 4._kp*ddNendddsigmastar+ &
        4._kp/3._kp*ddlnrhophiendddsigmastar

  end function MC4_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC4_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_rad_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: ddNendddphistar, ddrhophiendddphistar, drhophienddphistar, rhophiend, &
                ddlnrhophiendddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    ddlnrhophiendddphistar = ddrhophiendddphistar/rhophiend-(drhophienddphistar/rhophiend)**2

    MC4_ddlnrho_rad_dec_ddphistar = 4._kp*ddNendddphistar+ &
        4._kp/3._kp*ddlnrhophiendddphistar

  end function MC4_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC4_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: ddNenddphistardsigmastar, ddrhophienddphistardsigmastar, drhophienddphistar, &
        rhophiend, drhophienddsigmastar, ddlnrhophienddphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    ddlnrhophienddphistardsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
                                      drhophienddsigmastar*drhophienddphistar/rhophiend**2

    MC4_ddlnrho_rad_dec_dphistar_dsigmastar = 4._kp*ddNenddphistardsigmastar+ &
        4._kp/3._kp*ddlnrhophienddphistardsigmastar

  end function MC4_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC4_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_dlnrho_mat_dec_dsigmastar
    real(kp) :: rhophiend, drhophienddsigmastar, dlnrhophienddsigmastar, dNenddsigmastar, &
                dsigmaenddsigmastar, dlnsigmaenddsigmastar, Hend, dHenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dlnsigmaenddsigmastar = dsigmaenddsigmastar/sigmaend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC4_dlnrho_mat_dec_dsigmastar = 3._kp*dNenddsigmastar+dlnrhophienddsigmastar+ &
            2._kp*dlnsigmaenddsigmastar-4._kp*msigma**2*dHenddsigmastar/(9._kp*Hend**3)

  end function MC4_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC4_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_dlnrho_mat_dec_dphistar
    real(kp) :: rhophiend, drhophienddphistar, dlnrhophienddphistar, dNenddphistar, &
                dsigmaenddphistar, dlnsigmaenddphistar, Hend, dHenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    dlnrhophienddphistar = drhophienddphistar/rhophiend
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dlnsigmaenddphistar = dsigmaenddphistar/sigmaend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC4_dlnrho_mat_dec_dphistar = 3._kp*dNenddphistar+dlnrhophienddphistar+ &
            2._kp*dlnsigmaenddphistar-4._kp*msigma**2*dHenddphistar/(9._kp*Hend**3)

  end function MC4_dlnrho_mat_dec_dphistar


! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC4_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_mat_dec_ddphistar
    real(kp) :: rhophiend, drhophienddphistar, ddrhophiendddphistar, ddlnrhophiendddphistar, &
                ddNendddphistar, ddsigmaendddphistar, dsigmaenddphistar, &
                ddlnsigmaendddphistar, Hend, dHenddphistar, ddHendddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddphistar = ddrhophiendddphistar/rhophiend-(drhophienddphistar/rhophiend)**2
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddlnsigmaendddphistar = ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC4_ddlnrho_mat_dec_ddphistar = 3._kp*ddNendddphistar+ddlnrhophiendddphistar+ &
            2._kp*ddlnsigmaendddphistar-4._kp*msigma**2*ddHendddphistar/(9._kp*Hend**3)+ &
            4._kp*msigma**2*dHenddphistar**2/(Hend**4)

  end function MC4_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC4_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_mat_dec_ddsigmastar
    real(kp) :: rhophiend, drhophienddsigmastar, ddrhophiendddsigmastar, &
                ddlnrhophiendddsigmastar, ddNendddsigmastar, ddsigmaendddsigmastar, &
                dsigmaenddsigmastar, ddlnsigmaendddsigmastar, Hend, dHenddsigmastar, &
                ddHendddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddsigmastar = ddrhophiendddsigmastar/rhophiend-(drhophienddsigmastar/rhophiend)**2
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddlnsigmaendddsigmastar = ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)

    MC4_ddlnrho_mat_dec_ddsigmastar = 3._kp*ddNendddsigmastar+ddlnrhophiendddsigmastar+ &
            2._kp*ddlnsigmaendddsigmastar-4._kp*msigma**2*ddHendddsigmastar/(9._kp*Hend**3)+ &
            4._kp*msigma**2*dHenddsigmastar**2/(Hend**4)

  end function MC4_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC4_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp) :: rhophiend, drhophienddphistar, drhophienddsigmastar, &
                ddrhophienddphistardsigmastar, ddlnrhophienddphistardsigmastar, &
                ddNenddphistardsigmastar, ddsigmaenddphistardsigmastar, dsigmaenddphistar, &
                dsigmaenddsigmastar, ddlnsigmaenddphistardsigmastar, Hend, dHenddphistar, &
                dHenddsigmastar, ddHenddphistardsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophienddphistardsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
                                      drhophienddsigmastar*drhophienddphistar/rhophiend**2
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam,sigmaend, phistar, msigma)
    ddlnsigmaenddphistardsigmastar = ddsigmaenddphistardsigmastar/sigmaend- &
          dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)

    MC4_ddlnrho_mat_dec_dphistar_dsigmastar = 3._kp*ddNenddphistardsigmastar+ &
        ddlnrhophienddphistardsigmastar+ 2._kp*ddlnsigmaenddphistardsigmastar- &
        4._kp*msigma**2*ddHenddphistardsigmastar/(9._kp*Hend**3)+ &
        4._kp*msigma**2*dHenddphistar*dHenddsigmastar/(Hend**4)

  end function MC4_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC4_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC4_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC4_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC4_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC4_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC4_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC4_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC4_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC4_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC4_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC4_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC4_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC4_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC4_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC4_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC4_Pzeta_relativecontribution_sigma


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC4_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC4_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC4_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC4_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC4_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC4_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC4_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC4_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC4_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC4_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC4_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC4_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC4_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC4_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC4_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC4_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC4_lnRrad
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC4_lnRrad = MC1_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC4_lnRrad

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC5         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 2
  function MC5_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma
    real(kp) :: MC5_sigmaend_max


    MC5_sigmaend_max = sqrt(2._kp)*exp(2._kp/9._kp)

  end function MC5_sigmaend_max

!returns the minimum allowed value for sigmaend for case 2
  function MC5_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC5_sigmaend_min

    MC5_sigmaend_min = MC4_sigmaend_max(msigma,Gammaphi,Gammasigma)

  end function MC5_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! First equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of first equality between energy densities of phi and sigma
  function MC5_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC5_Heq, Hend, rhophiend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC5_Heq = 1.5_kp**2*exp(-8._kp/9._kp+4._kp*msigma**2/(9._kp*Hend**2))* &
                Gammaphi*sigmaend**4*Hend**4/(rhophiend**2)

  end function MC5_Heq

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC5_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC5_rhogamma_dec, Hend, rhophiend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC5_rhogamma_dec = (2._kp/3._kp)**(2._kp/3._kp)*exp(16._kp/27._kp-8._kp/27._kp* &
                        msigma**2/Hend**2)*(rhophiend/Hend**2)**(7._kp/3._kp)* &
                        (Gammasigma/sigmaend)**(8._kp/3._kp)*Gammaphi**(-2._kp/3._kp)

  end function MC5_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC5_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC5_rhomatter_dec, Hend, rhophiend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC5_rhomatter_dec = Gammasigma**2*rhophiend/Hend**2

  end function MC5_rhomatter_dec

  function MC5_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC5_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC5_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    RhoMatterDec = MC5_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC5_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC5_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC5_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_dlnrho_rad_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_dlnrho_rad_dec_dsigmastar = MC4_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC5_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_dlnrho_rad_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_dlnrho_rad_dec_dphistar = MC4_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC5_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_rad_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_rad_dec_ddsigmastar = MC4_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC5_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_rad_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_rad_dec_ddphistar = MC4_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC5_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_rad_dec_dphistar_dsigmastar = MC4_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, &
            Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
            sigmaend, phistar)

  end function MC5_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC5_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_dlnrho_mat_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_dlnrho_mat_dec_dsigmastar = MC4_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC5_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_dlnrho_mat_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_dlnrho_mat_dec_dphistar = MC4_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_dlnrho_mat_dec_dphistar


! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC5_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_mat_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_mat_dec_ddphistar = MC4_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC5_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_mat_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_mat_dec_ddsigmastar = MC4_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC5_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC5_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC5_ddlnrho_mat_dec_dphistar_dsigmastar = MC4_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, &
        Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, &
        phistar)

  end function MC5_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC5_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC5_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC5_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC5_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC5_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC5_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC5_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC5_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC5_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC5_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC5_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC5_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC5_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC5_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC5_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                        dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                        dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC5_Pzeta_relativecontribution_sigma


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC5_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC5_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC5_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC5_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC5_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC5_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC5_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC5_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC5_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC5_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC5_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC5_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC5_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC5_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC5_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC5_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC5_lnRrad, Hend, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC5_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC5_lnRrad = log(Gammaphi*Gammasigma/(Hend*Heq))/6._kp

  end function MC5_lnRrad


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC6         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  function MC6_sigmaend_max(V,phiend0,Vparam,msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC6_sigmaend_max, Vend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Vend = V(phiend0(Vparam),Vparam)
    MC6_sigmaend_max = sqrt(2._kp*Vend)/msigma

  end function MC6_sigmaend_max


!returns the minimum allowed value for sigmaend for case 6
  function MC6_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC6_sigmaend_min

    MC6_sigmaend_min = MC5_sigmaend_max(msigma,Gammaphi,Gammasigma)

  end function MC6_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! First equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of first equality between energy densities of phi and sigma
  function MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC6_Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC6_Heq

! Returns sigma at the equivalence time, as a function of Heq
  function MC6_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    real(kp) :: MC6_sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, Heq
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC6_sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, &
                  sigmaend, Heq)

  end function MC6_sigmaeq

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC6_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_rhogamma_dec, Hend, rhophiend, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC6_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    MC6_rhogamma_dec = 2._kp*(Gammasigma/Gammaphi)**(8._kp/3._kp)* &
        (Gammaphi*Heq/(msigma*Hend))**2*rhophiend*exp(1.5_kp-0.75_kp*sigmaeq**2)

  end function MC6_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC6_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_rhomatter_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC6_rhomatter_dec = 3._kp*Gammasigma**2

  end function MC6_rhomatter_dec

  function MC6_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC6_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi,&
                    Gammasigma, sigmaend)
    RhoMatterDec = MC6_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                    Gammasigma, sigmaend)
    MC6_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC6_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC6_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_dlnrho_rad_dec_dsigmastar
    real(kp) :: dNenddsigmastar, dlnrhophienddsigmastar, dlnHenddsigmastar, &
                dlnHeqdsigmastar, dlnsigmaeqdsigmastar, rhophiend, drhophienddsigmastar, &
                Hend, dHenddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHeqdsigmastar = MC3_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_dlnrho_rad_dec_dsigmastar = 4._kp*dNenddsigmastar+dlnrhophienddsigmastar+ &
        2._kp/3._kp*(dlnHenddsigmastar-dlnHeqdsigmastar)+ &
        0.5_kp*sigmaeq**2*dlnsigmaeqdsigmastar

  end function MC6_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC6_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_dlnrho_rad_dec_dphistar
    real(kp) :: dNenddphistar, dlnrhophienddphistar, dlnHenddphistar, &
                dlnHeqdphistar, dlnsigmaeqdphistar, rhophiend, drhophienddphistar, &
                Hend, dHenddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dlnrhophienddphistar = drhophienddphistar/rhophiend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHeqdphistar = MC3_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_dlnrho_rad_dec_dphistar = 4._kp*dNenddphistar+dlnrhophienddphistar+ &
        2._kp/3._kp*(dlnHenddphistar-dlnHeqdphistar)+ &
        0.5_kp*sigmaeq**2*dlnsigmaeqdphistar

  end function MC6_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC6_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_rad_dec_ddsigmastar
    real(kp) :: ddNendddsigmastar, ddlnrhophiendddsigmastar, ddlnHendddsigmastar, &
                ddlnHeqddsigmastar, dlnsigmaeqdsigmastar, ddlnsigmaeqddsigmastar, &
                rhophiend, drhophienddsigmastar, ddrhophiendddsigmastar, Hend, &
                dHenddsigmastar, ddHendddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddsigmastar = ddrhophiendddsigmastar/rhophiend-(drhophienddsigmastar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    ddlnHeqddsigmastar = MC3_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddsigmastar = MC3_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_rad_dec_ddsigmastar = 4._kp*ddNendddsigmastar+ddlnrhophiendddsigmastar+ &
        2._kp/3._kp*(ddlnHendddsigmastar-ddlnHeqddsigmastar)+ &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddsigmastar+dlnsigmaeqdsigmastar**2)

  end function MC6_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC6_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_rad_dec_ddphistar
    real(kp) :: ddNendddphistar, ddlnrhophiendddphistar, ddlnHendddphistar, &
                ddlnHeqddphistar, dlnsigmaeqdphistar, ddlnsigmaeqddphistar, &
                rhophiend, drhophienddphistar, ddrhophiendddphistar, Hend, &
                dHenddphistar, ddHendddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddphistar = ddrhophiendddphistar/rhophiend-(drhophienddphistar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    ddlnHeqddphistar = MC3_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddphistar = MC3_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_rad_dec_ddphistar = 4._kp*ddNendddphistar+ddlnrhophiendddphistar+ &
        2._kp/3._kp*(ddlnHendddphistar-ddlnHeqddphistar)+ &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddphistar+dlnsigmaeqdphistar**2)

  end function MC6_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC6_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp) :: ddNenddphistardsigmastar, ddlnrhophienddphistardsigmastar, &
                ddlnHenddphistardsigmastar, ddlnHeqdphistardsigmastar, &
                dlnsigmaeqdsigmastar, dlnsigmaeqdphistar, ddlnsigmaeqdphistardsigmastar, &
                rhophiend, drhophienddsigmastar, drhophienddphistar, &
                ddrhophienddphistardsigmastar, Hend, dHenddsigmastar, dHenddphistar, &
                ddHenddphistardsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, &
                phiend0, InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophienddphistardsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
                                      drhophienddsigmastar*drhophienddphistar/rhophiend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    ddlnHeqdphistardsigmastar = MC3_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqdphistardsigmastar = MC3_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_rad_dec_dphistar_dsigmastar = 4._kp*ddNenddphistardsigmastar+ &
        ddlnrhophienddphistardsigmastar+2._kp/3._kp*(ddlnHenddphistardsigmastar- &
        ddlnHeqdphistardsigmastar)+sigmaeq**2*(0.5_kp*ddlnsigmaeqdphistardsigmastar+ &
        dlnsigmaeqdsigmastar*dlnsigmaeqdphistar)

  end function MC6_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC6_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_dlnrho_mat_dec_dsigmastar
    real(kp) :: dNenddsigmastar, dlnHenddsigmastar, &
                dlnHeqdsigmastar, dlnsigmaeqdsigmastar, &
                Hend, dHenddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHeqdsigmastar = MC3_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_dlnrho_mat_dec_dsigmastar = 3._kp*dNenddsigmastar+ &
        2._kp*(dlnHenddsigmastar-dlnHeqdsigmastar)+ &
        1.5_kp*sigmaeq**2*dlnsigmaeqdsigmastar

  end function MC6_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC6_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_dlnrho_mat_dec_dphistar
    real(kp) :: dNenddphistar, dlnHenddphistar, &
                dlnHeqdphistar, dlnsigmaeqdphistar, &
                Hend, dHenddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHeqdphistar = MC3_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_dlnrho_mat_dec_dphistar = 3._kp*dNenddphistar+ &
        2._kp*(dlnHenddphistar-dlnHeqdphistar)+ &
        1.5_kp*sigmaeq**2*dlnsigmaeqdphistar

  end function MC6_dlnrho_mat_dec_dphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC6_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_mat_dec_ddsigmastar
    real(kp) :: ddNendddsigmastar, ddlnHendddsigmastar, &
                ddlnHeqddsigmastar, dlnsigmaeqdsigmastar, ddlnsigmaeqddsigmastar, &
                Hend, dHenddsigmastar, ddHendddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    ddlnHeqddsigmastar = MC3_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddsigmastar = MC3_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_mat_dec_ddsigmastar = 3._kp*ddNendddsigmastar+ &
        2._kp*(ddlnHendddsigmastar-ddlnHeqddsigmastar)+3._kp* &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddsigmastar+dlnsigmaeqdsigmastar**2)

  end function MC6_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC6_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_mat_dec_ddphistar
    real(kp) :: ddNendddphistar, ddlnHendddphistar, &
                ddlnHeqddphistar, dlnsigmaeqdphistar, ddlnsigmaeqddphistar, &
                Hend, dHenddphistar, ddHendddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    ddlnHeqddphistar = MC3_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddphistar = MC3_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_mat_dec_ddphistar = 3._kp*ddNendddphistar+ &
        2._kp*(ddlnHendddphistar-ddlnHeqddphistar)+3._kp* &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddphistar+dlnsigmaeqdphistar**2)

  end function MC6_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC6_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp) :: ddNenddphistardsigmastar, &
                ddlnHenddphistardsigmastar, ddlnHeqdphistardsigmastar, &
                dlnsigmaeqdsigmastar, dlnsigmaeqdphistar, ddlnsigmaeqdphistardsigmastar, &
                Hend, dHenddsigmastar, dHenddphistar, &
                ddHenddphistardsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    ddlnHeqdphistardsigmastar = MC3_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC6_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC3_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnsigmaeqdphistar = MC3_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqdphistardsigmastar = MC3_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_ddlnrho_mat_dec_dphistar_dsigmastar = 3._kp*ddNenddphistardsigmastar+ &
        2._kp*(ddlnHenddphistardsigmastar- &
        ddlnHeqdphistardsigmastar)+sigmaeq**2*3._kp*(0.5_kp*ddlnsigmaeqdphistardsigmastar+ &
        dlnsigmaeqdsigmastar*dlnsigmaeqdphistar)

  end function MC6_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC6_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC6_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC6_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC6_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC6_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC6_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC6_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC6_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC6_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC6_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC6_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC6_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC6_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC6_Pzeta_relativecontribution_sigma


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC6_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC6_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC6_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC6_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC6_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC6_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC6_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC6_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC6_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC6_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC6_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC6_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC6_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC6_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC6_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC6_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_lnRrad
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC6_lnRrad = MC3_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC6_lnRrad

! Returns the minimum value of M so that the CMB pivot scale crosses the Hubble radius during the first inflationary period. WARNING: This function must be called with M=1 in the Vparam argument.
  function MC6_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC6_Mmin_reh
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC6_Mmin_reh = MC3_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                Gammasigma, sigmaend)

  end function MC6_Mmin_reh


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC7         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  function MC7_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC7_sigmaend_max

    MC7_sigmaend_max = 2._kp**(5._kp/8._kp)*(Gammasigma/msigma)**(0.25_kp)* &
                        exp(1._kp/3._kp)

  end function MC7_sigmaend_max

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC7_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC7_rhogamma_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC7_rhogamma_dec = 3._kp*Gammasigma**2

  end function MC7_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC7_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC7_rhomatter_dec, Hend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC7_rhomatter_dec = 0.75_kp*msigma**2*sigmaend**2* &
            (Gammasigma*sqrt(2._kp)/msigma)**(1.5_kp)* &
            exp(-2._kp/3._kp-msigma**2/(18._kp*Gammaphi**2)+ &
            2._kp*msigma**2/(9._kp*Hend**2))

  end function MC7_rhomatter_dec

  function MC7_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC7_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC7_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi,&
                    Gammasigma, sigmaend)
    RhoMatterDec = MC7_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                    Gammasigma, sigmaend)
    MC7_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC7_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_dlnrho_rad_dec_dsigmastar
    real(kp) :: dNenddsigmastar, dlnHenddsigmastar, Hend, dHenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend

    MC7_dlnrho_rad_dec_dsigmastar = 4._kp*dNenddsigmastar+8._kp/3._kp*dlnHenddsigmastar

  end function MC7_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_dlnrho_rad_dec_dphistar
    real(kp) :: dNenddphistar, dlnHenddphistar, Hend, dHenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend

    MC7_dlnrho_rad_dec_dphistar = 4._kp*dNenddphistar+8._kp/3._kp*dlnHenddphistar

  end function MC7_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC7_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_rad_dec_ddsigmastar
    real(kp) :: ddNendddsigmastar, ddlnHendddsigmastar, Hend, &
                dHenddsigmastar, ddHendddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2

    MC7_ddlnrho_rad_dec_ddsigmastar = 4._kp*ddNendddsigmastar+8._kp/3._kp*ddlnHendddsigmastar

  end function MC7_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC7_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_rad_dec_ddphistar
    real(kp) :: ddNendddphistar, ddlnHendddphistar, Hend, &
                dHenddphistar, ddHendddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2

    MC7_ddlnrho_rad_dec_ddphistar = 4._kp*ddNendddphistar+8._kp/3._kp*ddlnHendddphistar

  end function MC7_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC7_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp) :: ddNenddphistardsigmastar, ddlnHenddphistardsigmastar, Hend, &
                dHenddphistar, dHenddsigmastar, ddHenddphistardsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2

    MC7_ddlnrho_rad_dec_dphistar_dsigmastar = 4._kp*ddNenddphistardsigmastar+ &
        8._kp/3._kp*ddlnHenddphistardsigmastar

  end function MC7_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC7_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_dlnrho_mat_dec_dsigmastar
    real(kp) :: dNenddsigmastar, dlnHenddsigmastar, &
                Hend, dHenddsigmastar, dlnsigmaenddsigmastar, dsigmaenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dlnsigmaenddsigmastar = dsigmaenddsigmastar/sigmaend

    MC7_dlnrho_mat_dec_dsigmastar = 3._kp*dNenddsigmastar+2._kp*dlnsigmaenddsigmastar+ &
            (2._kp-4._kp*msigma**2/(9._kp*Hend**2))*dlnHenddsigmastar

  end function MC7_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC7_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_dlnrho_mat_dec_dphistar
    real(kp) :: dNenddphistar, dlnHenddphistar, &
                Hend, dHenddphistar, dlnsigmaenddphistar, dsigmaenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dlnsigmaenddphistar = dsigmaenddphistar/sigmaend

    MC7_dlnrho_mat_dec_dphistar = 3._kp*dNenddphistar+2._kp*dlnsigmaenddphistar+ &
            (2._kp-4._kp*msigma**2/(9._kp*Hend**2))*dlnHenddphistar

  end function MC7_dlnrho_mat_dec_dphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC7_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_mat_dec_ddsigmastar
    real(kp) :: ddNendddsigmastar, ddlnHendddsigmastar, Hend, dHenddsigmastar, &
                ddHendddsigmastar, ddlnsigmaendddsigmastar, dsigmaenddsigmastar, &
                ddsigmaendddsigmastar, dlnHenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam,sigmaend, phistar, msigma)
    ddlnsigmaendddsigmastar = ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/sigmaend**2

    MC7_ddlnrho_mat_dec_ddsigmastar = 3._kp*ddNendddsigmastar+2._kp*ddlnsigmaendddsigmastar+ &
            (2._kp-4._kp*msigma**2/(9._kp*Hend**2))*ddlnHendddsigmastar+ &
            8._kp*msigma**2/(9._kp*Hend**2)*dlnHenddsigmastar**2

  end function MC7_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC7_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_mat_dec_ddphistar
    real(kp) :: ddNendddphistar, ddlnHendddphistar, Hend, dHenddphistar, &
                ddHendddphistar, ddlnsigmaendddphistar, dsigmaenddphistar, &
                ddsigmaendddphistar, dlnHenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam,sigmaend, phistar, msigma)
    ddlnsigmaendddphistar = ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/sigmaend**2

    MC7_ddlnrho_mat_dec_ddphistar = 3._kp*ddNendddphistar+2._kp*ddlnsigmaendddphistar+ &
            (2._kp-4._kp*msigma**2/(9._kp*Hend**2))*ddlnHendddphistar+ &
            8._kp*msigma**2/(9._kp*Hend**2)*dlnHenddphistar**2

  end function MC7_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC7_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp) :: ddNenddphistardsigmastar, ddlnHenddphistardsigmastar, Hend, dHenddphistar, &
                dHenddsigmastar, ddHenddphistardsigmastar, ddlnsigmaenddphistardsigmastar, &
                dsigmaenddphistar, dsigmaenddsigmastar, ddsigmaenddphistardsigmastar, &
                dlnHenddphistar, dlnHenddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam,sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam,sigmaend, phistar, msigma)
    ddlnsigmaenddphistardsigmastar = ddsigmaenddphistardsigmastar/sigmaend- &
          dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2


    MC7_ddlnrho_mat_dec_dphistar_dsigmastar = 3._kp*ddNenddphistardsigmastar+ &
        2._kp*ddlnsigmaenddphistardsigmastar+ &
        (2._kp-4._kp*msigma**2/(9._kp*Hend**2))*ddlnHenddphistardsigmastar+ &
        8._kp*msigma**2/(9._kp*Hend**2)*dlnHenddphistar*dlnHenddsigmastar

  end function MC7_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC7_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC7_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC7_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC7_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC7_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC7_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC7_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC7_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC7_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC7_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC7_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                        dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                        dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC7_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC7_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
        Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC7_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, &
        dlnrhomat_dphirad, dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC7_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC7_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC7_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC7_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC7_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC7_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC7_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC7_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC7_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC7_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
                ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, &
                dlnrhomat_dphirad, dlnrhomat_dphimat, ddlnrhomat_ddphirad, &
                ddlnrhomat_ddphimat, ddlnrhomat_dphirad_dphimat)

  end function MC7_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC7_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC7_lnRrad
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC7_lnRrad = MC1_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC7_lnRrad


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC8         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  function MC8_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC8_sigmaend_min

    MC8_sigmaend_min = MC7_sigmaend_max(msigma,Gammaphi,Gammasigma)

  end function MC8_sigmaend_min

  function MC8_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC8_sigmaend_max

    MC8_sigmaend_max = sqrt(2._kp)*exp(1._kp/3._kp)

  end function MC8_sigmaend_max

!!!!!!!!!!!!!!!!!!!!!!!!!
!! equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of equality between energy densities of phi and sigma
  function MC8_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC8_Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    real(kp) :: Hend, rhophiend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    type(transfert) :: MCData
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC8_Heq = 9._kp/(2._kp**(2.5_kp))*exp(-4._kp/3._kp-msigma**2/(9._kp*Gammaphi**2)+ &
              4._kp*msigma**2/(9._kp*Hend**2))*msigma*sigmaend**4*Hend**4/rhophiend**2

  end function MC8_Heq


!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC8_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC8_rhogamma_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    real(kp) :: Hend, rhophiend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC8_rhogamma_dec = 2._kp**(5._kp/3._kp)*3._kp**(-4._kp/3._kp)* &
            exp(8._kp/9._kp+2._kp*msigma**2/(27._kp*Gammaphi**2)- &
            8._kp*msigma**2/(27._kp*Hend**2))*(rhophiend/Hend**2)**(7._kp/3._kp)* &
            (Gammasigma/sigmaend)**(8._kp/3._kp)*msigma**(-2._kp/3._kp)

  end function MC8_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC8_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC8_rhomatter_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    real(kp) :: Hend, rhophiend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC8_rhomatter_dec = Gammasigma**2*rhophiend/Hend**2

  end function MC8_rhomatter_dec

  function MC8_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC8_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC8_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi,&
                    Gammasigma, sigmaend)
    RhoMatterDec = MC8_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                    Gammasigma, sigmaend)
    MC8_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC8_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC8_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_dlnrho_rad_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_dlnrho_rad_dec_dsigmastar = MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC8_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_dlnrho_rad_dec_dphistar
    real(kp) :: dNenddphistar, dlnHenddphistar, Hend, dHenddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_dlnrho_rad_dec_dphistar = MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC8_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_rad_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_rad_dec_ddsigmastar = MC7_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC8_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_rad_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_rad_dec_ddphistar = MC7_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC8_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_rad_dec_dphistar_dsigmastar = MC7_ddlnrho_rad_dec_dphistar_dsigmastar(V, &
        Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
        sigmaend, phistar)

  end function MC8_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC8_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_dlnrho_mat_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_dlnrho_mat_dec_dsigmastar = MC7_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC8_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_dlnrho_mat_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_dlnrho_mat_dec_dphistar = MC7_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_dlnrho_mat_dec_dphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC8_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_mat_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_mat_dec_ddsigmastar = MC7_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC8_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_mat_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_mat_dec_ddphistar = MC7_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC8_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC8_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC8_ddlnrho_mat_dec_dphistar_dsigmastar = MC7_ddlnrho_mat_dec_dphistar_dsigmastar(V, &
        Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
        sigmaend, phistar)

  end function MC8_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC8_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC8_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC8_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC8_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC8_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC8_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC8_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC8_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC8_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC8_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC8_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC8_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC8_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC8_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC8_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                    dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                    dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC8_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC8_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
        Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC8_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, &
        dlnrhomat_dphirad, dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC8_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC8_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC8_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC8_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC8_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC8_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC8_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC8_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC8_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC8_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC8_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC8_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
                ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, &
                dlnrhomat_dphirad, dlnrhomat_dphimat, ddlnrhomat_ddphirad, &
                ddlnrhomat_ddphimat, ddlnrhomat_dphirad_dphimat)

  end function MC8_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC8_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC8_lnRrad
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    real(kp) :: Hend, Heq
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC8_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    MC8_lnRrad = log(Gammaphi*Gammasigma/(Hend*Heq))/6._kp

  end function MC8_lnRrad

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!         MC9         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!returns the maximum allowed value for sigmaend for case 9
  function MC9_sigmaend_max(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC9_sigmaend_max

    MC9_sigmaend_max = sqrt(6._kp)*Gammaphi/msigma

  end function MC9_sigmaend_max

!returns the minimum allowed value for sigmaend for case 9
  function MC9_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC9_sigmaend_min

    MC9_sigmaend_min = sqrt(3._kp)

  end function MC9_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! First equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of first equality between energy densities of phi and sigma
  function MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    real(kp) :: Hend, rhophiend, xmini, xmaxi, x ! x is (Heq/msigma)^2
    type(transfert) :: MCData
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MCData%real1 = msigma
    MCData%real2 = Gammaphi
    MCData%real3 = Gammasigma
    MCData%real4 = sigmaend

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MCData%real5 = Hend
    MCData%real6 = rhophiend

    xmini = 1._kp/9._kp ! where find_MC9_Heq is maximal
    xmaxi = (Gammaphi/msigma)**2*5._kp ! 5 is a safety number

    x = zbrent(find_MC9_Heq,xmini,xmaxi,tolFind,MCData)

    MC9_Heq = msigma*sqrt(x)

  end function MC9_Heq

! Returns the equation one must solve to obtain Heq1
  function find_MC9_Heq(x,MCData)
    implicit none
    real(kp) :: find_MC9_Heq
    real(kp), intent(in) :: x ! x is (Heq/msigma)^2
    type(transfert), optional, intent(inout) :: MCData
    real(kp) :: mphi, msigma, Gammaphi, Gammasigma, sigmaend, Hend, rhophiend

    msigma = MCData%real1
    Gammaphi = MCData%real2
    Gammasigma = MCData%real3
    sigmaend = MCData%real4
    Hend = MCData%real5
    rhophiend = MCData%real6

    find_MC9_Heq = 0.5_kp*sigmaend**2*Hend**2/rhophiend*exp(-1._kp/(6._kp*x)+ &
                     2._kp*msigma**2/(9._kp*Hend**2)-msigma**2/(18._kp*Gammaphi**2))* &
                    (1._kp+1._kp/(9._kp*x))/x-1._kp

  end function find_MC9_Heq

! Returns sigma at the first equivalence time, as a function of Heq
  function MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    real(kp) :: MC9_sigmaeq, Hend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, Heq
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    MC9_sigmaeq = sigmaend*exp(-msigma**2/3._kp*(0.25_kp/(Heq**2)+ &
                1._kp/(12._kp*Gammaphi**2)-1._kp/(3._kp*Hend**2)))

  end function MC9_sigmaeq

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (sigma decay products) at last decay time
  function MC9_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_rhogamma_dec
    real(kp) :: Heq, sigmaeq, Hend, rhophiend
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Heq = MC9_Heq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)

    MC9_rhogamma_dec = rhophiend*(Heq/Hend)**2*exp(2._kp-sigmaeq**2)* &
                        (Gammasigma*sqrt(2._kp)/msigma)**(8._kp/3._kp)

  end function MC9_rhogamma_dec

! Energy density of matter (phi) at last decay time
  function MC9_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_rhomatter_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC9_rhomatter_dec = 3._kp*Gammasigma**2

  end function MC9_rhomatter_dec

  function MC9_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC9_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                   Gammasigma, sigmaend)
    RhoMatterDec = MC9_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                   Gammasigma, sigmaend)
    MC9_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC9_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!   d Heq / d sigma* and phi*   !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnHeq / d phistar
  function MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnHeq_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    real(kp) :: dsigmaenddphistar, dlnsigmaenddphistar, drhophienddphistar, &
                dlnrhophienddphistar, dHenddphistar, dlnHenddphistar, rhophiend, Hend, Heq
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, sigmaend, phistar, msigma)
    dlnsigmaenddphistar = dsigmaenddphistar/sigmaend
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                         Vparam, sigmaend, phistar, msigma)
    dlnrhophienddphistar = drhophienddphistar/rhophiend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_dlnHeq_dphistar = (dlnsigmaenddphistar+(1._kp-2._kp*msigma**2/(9._kp*Hend**2))* &
                            dlnHenddphistar-0.5_kp*dlnrhophienddphistar)/ &
                            (1._kp+msigma**2/(9._kp*Heq**2+msigma**2)- &
                            msigma**2/(6._kp*Heq**2))

  end function MC9_dlnHeq_dphistar

! Returns d^2 lnHeq / d phistar ^2
  function MC9_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnHeq_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, ddsigmaendddphistar, ddlnsigmaendddphistar, &
                rhophiend, drhophienddphistar, ddrhophiendddphistar, ddlnrhophiendddphistar, &
                Hend, dHenddphistar, ddHendddphistar, dlnHenddphistar, ddlnHendddphistar, &
                Heq, dlnHeqdphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddphistar = ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/sigmaend**2
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    ddrhophiendddphistar = MC_ddrho_phi_end_ddphistar(V, Vp, Vpp, phiend0, &
                            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddphistar = ddrhophiendddphistar/rhophiend-(drhophienddphistar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend-(dHenddphistar/Hend)**2
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnHeqdphistar = MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnHeq_ddphistar = (ddlnsigmaendddphistar-0.5_kp*ddlnrhophiendddphistar+ &
            (1._kp-2._kp*msigma**2/(9._kp*Hend**2))*ddlnHendddphistar+ &
            4._kp*msigma**2/(9._kp*Hend**2)*dlnHenddphistar**2-2._kp* &
            dlnHeqdphistar**2*(msigma**2/(6._kp*Heq**2)-9._kp*Heq**2*msigma**2/ &
            (9._kp*Heq**2+msigma**2)**2))/(1._kp+msigma**2/(9._kp*Heq**2+msigma**2)- &
            msigma**2/(6._kp*Heq**2))


  end function MC9_ddlnHeq_ddphistar


! Returns d lnHeq / d sigmastar
  function MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnHeq_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    real(kp) :: dsigmaenddsigmastar, dlnsigmaenddsigmastar, drhophienddsigmastar, &
                dlnrhophienddsigmastar, dHenddsigmastar, dlnHenddsigmastar, &
                rhophiend, Hend, Heq
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, sigmaend, phistar, msigma)
    dlnsigmaenddsigmastar = dsigmaenddsigmastar/sigmaend
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    dlnrhophienddsigmastar = drhophienddsigmastar/rhophiend
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_dlnHeq_dsigmastar = (dlnsigmaenddsigmastar+(1._kp-2._kp*msigma**2/(9._kp*Hend**2))* &
                            dlnHenddsigmastar-0.5_kp*dlnrhophienddsigmastar)/ &
                            (1._kp+msigma**2/(9._kp*Heq**2+msigma**2)- &
                            msigma**2/(6._kp*Heq**2))

  end function MC9_dlnHeq_dsigmastar

! Returns d^2 lnHeq / d sigmastar ^2
  function MC9_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnHeq_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, ddsigmaendddsigmastar, ddlnsigmaendddsigmastar, &
                rhophiend, drhophienddsigmastar, ddrhophiendddsigmastar, &
                ddlnrhophiendddsigmastar, Hend, dHenddsigmastar, ddHendddsigmastar, &
                dlnHenddsigmastar, ddlnHendddsigmastar, &
                Heq, dlnHeqdsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddsigmastar = ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/sigmaend**2
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophiendddsigmastar = MC_ddrho_phi_end_ddsigmastar(V, Vp, Vpp, phiend0, &
                            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophiendddsigmastar = ddrhophiendddsigmastar/rhophiend- &
                (drhophienddsigmastar/rhophiend)**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend-(dHenddsigmastar/Hend)**2
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnHeq_ddsigmastar = (ddlnsigmaendddsigmastar-0.5_kp*ddlnrhophiendddsigmastar+ &
            (1._kp-2._kp*msigma**2/(9._kp*Hend**2))*ddlnHendddsigmastar+ &
            4._kp*msigma**2/(9._kp*Hend**2)*dlnHenddsigmastar**2-2._kp* &
            dlnHeqdsigmastar**2*(msigma**2/(6._kp*Heq**2)-9._kp*Heq**2*msigma**2/ &
            (9._kp*Heq**2+msigma**2)**2))/(1._kp+msigma**2/(9._kp*Heq**2+msigma**2)- &
            msigma**2/(6._kp*Heq**2))


  end function MC9_ddlnHeq_ddsigmastar

! Returns d^2 lnHeq / d phistar d sigmastar
  function MC9_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnHeq_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, dsigmaenddsigmastar, ddsigmaenddphistardsigmastar, &
                ddlnsigmaenddphistardsigmastar, rhophiend, drhophienddphistar, &
                drhophienddsigmastar, ddrhophienddphistardsigmastar, &
                ddlnrhophienddphistardsigmastar, &
                Hend, dHenddphistar, dHenddsigmastar, ddHenddphistardsigmastar, &
                dlnHenddsigmastar, dlnHenddphistar, ddlnHenddphistardsigmastar, &
                Heq, dlnHeqdphistar, dlnHeqdsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, sigmaend, phistar, msigma)
    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnsigmaenddphistardsigmastar = ddsigmaenddphistardsigmastar/sigmaend- &
          dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    drhophienddphistar = MC_drho_phi_end_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, sigmaend, phistar, msigma)
    drhophienddsigmastar = MC_drho_phi_end_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddrhophienddphistardsigmastar = MC_ddrho_phi_end_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
                            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnrhophienddphistardsigmastar = ddrhophienddphistardsigmastar/rhophiend- &
                                      drhophienddsigmastar*drhophienddphistar/rhophiend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                                dHenddphistar*dHenddsigmastar/(Hend**2)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnHeqdphistar = MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnHeq_dphistar_dsigmastar = (ddlnsigmaenddphistardsigmastar-0.5_kp* &
            ddlnrhophienddphistardsigmastar+ &
            (1._kp-2._kp*msigma**2/(9._kp*Hend**2))*ddlnHenddphistardsigmastar+ &
            4._kp*msigma**2/(9._kp*Hend**2)*dlnHenddphistar*dlnHenddsigmastar-2._kp* &
            dlnHeqdphistar*dlnHeqdsigmastar*(msigma**2/(6._kp*Heq**2)-9._kp*Heq**2*msigma**2/ &
            (9._kp*Heq**2+msigma**2)**2))/(1._kp+msigma**2/(9._kp*Heq**2+msigma**2)- &
            msigma**2/(6._kp*Heq**2))

  end function MC9_ddlnHeq_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  d sigmaeq / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d ln sigmaeq / d phistar
  function MC9_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnsigmaeq_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, dHenddphistar, dlnHenddphistar, Hend, Heq, dlnHeqdphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, sigmaend, phistar, msigma)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHeqdphistar =  MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_dlnsigmaeq_dphistar = dsigmaenddphistar/sigmaend-2._kp*msigma**2/(9._kp*Hend**2)* &
                                dlnHenddphistar+msigma**2/(6._kp*Heq**2)*dlnHeqdphistar

  end function MC9_dlnsigmaeq_dphistar

! Returns d^2 ln sigmaeq / d phistar ^2
  function MC9_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
               msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnsigmaeq_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddphistar, ddsigmaendddphistar, ddlnsigmaendddphistar, dHenddphistar, &
                ddHendddphistar, dlnHenddphistar, ddlnHendddphistar, Hend, Heq, &
                dlnHeqdphistar, ddlnHeqddphistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    ddsigmaendddphistar = MC_ddsigmaend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddphistar = ddsigmaendddphistar/sigmaend-dsigmaenddphistar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                     phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    dlnHeqdphistar =  MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeqddphistar = MC9_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_ddlnsigmaeq_ddphistar = ddlnsigmaendddphistar-2._kp*msigma**2/(9._kp*Hend**2)* &
                                 (ddlnHendddphistar-2._kp*dlnHenddphistar**2)+ &
                                  msigma**2/(6._kp*Heq**2)*(ddlnHeqddphistar- &
                                  2._kp*dlnHeqdphistar**2)

  end function MC9_ddlnsigmaeq_ddphistar

! Returns d ln sigmaeq / d sigmastar
  function MC9_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnsigmaeq_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, dHenddsigmastar, dlnHenddsigmastar, Hend, Heq, &
                dlnHeqdsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_dlnsigmaeq_dsigmastar = dsigmaenddsigmastar/sigmaend-2._kp*msigma**2/ &
                (9._kp*Hend**2)*dlnHenddsigmastar+msigma**2/(6._kp*Heq**2)*dlnHeqdsigmastar

  end function MC9_dlnsigmaeq_dsigmastar

! Returns d^2 ln sigmaeq / d sigmastar ^2
  function MC9_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
              msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnsigmaeq_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, ddsigmaendddsigmastar, ddlnsigmaendddsigmastar, &
                dHenddsigmastar, ddHendddsigmastar, dlnHenddsigmastar, ddlnHendddsigmastar, &
                Hend, Heq, dlnHeqdsigmastar, ddlnHeqddsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    ddsigmaendddsigmastar = MC_ddsigmaend_ddsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnsigmaendddsigmastar = ddsigmaendddsigmastar/sigmaend-dsigmaenddsigmastar**2/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeqddsigmastar = MC9_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_ddlnsigmaeq_ddsigmastar = ddlnsigmaendddsigmastar-2._kp*msigma**2/(9._kp*Hend**2)* &
                                 (ddlnHendddsigmastar-2._kp*dlnHenddsigmastar**2)+ &
                                  msigma**2/(6._kp*Heq**2)*(ddlnHeqddsigmastar- &
                                  2._kp*dlnHeqdsigmastar**2)

  end function MC9_ddlnsigmaeq_ddsigmastar

! Returns d^2 ln sigmaeq / d phistar d sigmastar
  function MC9_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnsigmaeq_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: dsigmaenddsigmastar, dsigmaenddphistar, ddsigmaenddphistardsigmastar, &
                ddlnsigmaenddphistardsigmastar, dHenddsigmastar, dHenddphistar, &
                ddHenddphistardsigmastar, dlnHenddsigmastar, dlnHenddphistar, &
                ddlnHenddphistardsigmastar, Hend, Heq, dlnHeqdsigmastar, dlnHeqdphistar, &
                ddlnHeqdphistardsigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dsigmaenddsigmastar = MC_dsigmaend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
          Vparam,sigmaend, phistar, msigma)
    dsigmaenddphistar = MC_dsigmaend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    ddsigmaenddphistardsigmastar = MC_ddsigmaend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    ddlnsigmaenddphistardsigmastar = ddsigmaenddphistardsigmastar/sigmaend- &
          dsigmaenddsigmastar*dsigmaenddphistar/sigmaend**2
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                    phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
          InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHenddphistar = dHenddphistar/Hend
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend - &
               dHenddsigmastar*dHenddphistar/Hend**2
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnHeqdphistar =  MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
                       msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnHeqdphistardsigmastar = MC9_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
              InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

    MC9_ddlnsigmaeq_dphistar_dsigmastar = ddlnsigmaenddphistardsigmastar-2._kp*msigma**2/ &
        (9._kp*Hend**2)*(ddlnHenddphistardsigmastar-2._kp* &
        dlnHenddsigmastar*dlnHenddphistar)+msigma**2/ &
        (6._kp*Heq**2)*(ddlnHeqdphistardsigmastar- &
        2._kp*dlnHeqdsigmastar*dlnHeqdphistar)

  end function MC9_ddlnsigmaeq_dphistar_dsigmastar


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC9_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnrho_rad_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC9_dlnrho_rad_dec_dsigmastar = MC7_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC9_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC9_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnrho_rad_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC9_dlnrho_rad_dec_dphistar = MC7_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC9_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC9_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_rad_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC9_ddlnrho_rad_dec_ddsigmastar = MC7_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC9_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC9_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_rad_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC9_ddlnrho_rad_dec_ddphistar = MC7_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC9_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC9_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC9_ddlnrho_rad_dec_dphistar_dsigmastar = MC7_ddlnrho_rad_dec_dphistar_dsigmastar(V, &
            Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
            Gammasigma, sigmaend, phistar)

  end function MC9_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC9_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnrho_mat_dec_dsigmastar
    real(kp) :: dNenddsigmastar, dlnHenddsigmastar, &
                dlnHeqdsigmastar, dlnsigmaeqdsigmastar, &
                Hend, dHenddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddsigmastar = MC_dNend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddsigmastar = dHenddsigmastar/Hend
    dlnHeqdsigmastar = MC9_dlnHeq_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC9_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_dlnrho_mat_dec_dsigmastar = 3._kp*dNenddsigmastar+ &
        2._kp*dlnHenddsigmastar-1.5_kp*dlnHeqdsigmastar+ &
        1.5_kp*sigmaeq**2*dlnsigmaeqdsigmastar

  end function MC9_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC9_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
            msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_dlnrho_mat_dec_dphistar
    real(kp) :: dNenddphistar, dlnHenddphistar, &
                dlnHeqdphistar, dlnsigmaeqdphistar, &
                Hend, dHenddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    dNenddphistar = MC_dNend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
          sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dlnHenddphistar = dHenddphistar/Hend
    dlnHeqdphistar = MC9_dlnHeq_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC9_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_dlnrho_mat_dec_dphistar = 3._kp*dNenddphistar+ &
        2._kp*dlnHenddphistar-1.5_kp*dlnHeqdphistar+ &
        1.5_kp*sigmaeq**2*dlnsigmaeqdphistar

  end function MC9_dlnrho_mat_dec_dphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC9_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_mat_dec_ddsigmastar
    real(kp) :: ddNendddsigmastar, ddlnHendddsigmastar, &
                ddlnHeqddsigmastar, dlnsigmaeqdsigmastar, ddlnsigmaeqddsigmastar, &
                Hend, dHenddsigmastar, ddHendddsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddsigmastar = MC_ddNend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddsigmastar = MC_ddHend_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddsigmastar = ddHendddsigmastar/Hend - (dHenddsigmastar/Hend)**2
    ddlnHeqddsigmastar = MC9_ddlnHeq_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC9_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddsigmastar = MC9_ddlnsigmaeq_ddsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnrho_mat_dec_ddsigmastar = 3._kp*ddNendddsigmastar+ &
        2._kp*ddlnHendddsigmastar-1.5_kp*ddlnHeqddsigmastar+3._kp* &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddsigmastar+dlnsigmaeqdsigmastar**2)

  end function MC9_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC9_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, &
          Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_mat_dec_ddphistar
    real(kp) :: ddNendddphistar, ddlnHendddphistar, &
                ddlnHeqddphistar, dlnsigmaeqdphistar, ddlnsigmaeqddphistar, &
                Hend, dHenddphistar, ddHendddphistar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNendddphistar = MC_ddNend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHendddphistar = MC_ddHend_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddlnHendddphistar = ddHendddphistar/Hend - (dHenddphistar/Hend)**2
    ddlnHeqddphistar = MC9_ddlnHeq_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
             Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdphistar = MC9_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqddphistar = MC9_ddlnsigmaeq_ddphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnrho_mat_dec_ddphistar = 3._kp*ddNendddphistar+ &
        2._kp*ddlnHendddphistar-1.5_kp*ddlnHeqddphistar+3._kp* &
        sigmaeq**2*(0.5_kp*ddlnsigmaeqddphistar+dlnsigmaeqdphistar**2)

  end function MC9_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC9_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp) :: ddNenddphistardsigmastar, &
                ddlnHenddphistardsigmastar, ddlnHeqdphistardsigmastar, &
                dlnsigmaeqdsigmastar, dlnsigmaeqdphistar, ddlnsigmaeqdphistardsigmastar, &
                Hend, dHenddsigmastar, dHenddphistar, &
                ddHenddphistardsigmastar, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    ddNenddphistardsigmastar = MC_ddNend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, sigmaend, phistar, msigma)
    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    dHenddsigmastar = MC_dHend_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    dHenddphistar = MC_dHend_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, phistar, msigma, sigmaend)
    ddHenddphistardsigmastar = MC_ddHend_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, phistar, msigma, sigmaend)
    ddlnHenddphistardsigmastar = ddHenddphistardsigmastar/Hend- &
                dHenddphistar*dHenddsigmastar/Hend**2
    ddlnHeqdphistardsigmastar = MC9_ddlnHeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    dlnsigmaeqdsigmastar = MC9_dlnsigmaeq_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnsigmaeqdphistar = MC9_dlnsigmaeq_dphistar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnsigmaeqdphistardsigmastar = MC9_ddlnsigmaeq_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
                InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_ddlnrho_mat_dec_dphistar_dsigmastar = 3._kp*ddNenddphistardsigmastar+ &
        2._kp*ddlnHenddphistardsigmastar-1.5_kp* &
        ddlnHeqdphistardsigmastar+sigmaeq**2*3._kp*(0.5_kp*ddlnsigmaeqdphistardsigmastar+ &
        dlnsigmaeqdsigmastar*dlnsigmaeqdphistar)

  end function MC9_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC9_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC9_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC9_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC9_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC9_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC9_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC9_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC9_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                    Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC9_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC9_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC9_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC9_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC9_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                    dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                    dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC9_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC9_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC9_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC9_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC9_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC9_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC9_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC9_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC9_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC9_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC9_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC9_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC9_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC9_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC9_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC9_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC9_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_lnRrad
    real(kp) :: Hend, Heq, sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC9_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC9_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    MC9_lnRrad = log(Gammaphi*sqrt(2._kp)*Gammasigma/(Hend*msigma))/6._kp+ &
                 0.5_kp-0.25_kp*sigmaeq**2+0.5_kp*log(sqrt(2._kp)*Heq/msigma)

  end function MC9_lnRrad

! Returns the minimum value of M so that the CMB pivot scale crosses the Hubble radius during the first inflationary period. WARNING: This function must be called with M=1 in the Vparam argument.
  function MC9_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC9_Mmin_reh
    real(kp) :: phiend0val, Vend0
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    phiend0val = phiend0(Vparam)
    Vend0 = V(phiend0val,Vparam) ! This is Vend0/M^4, this function must be called with M=1 in the Vparam argument

    MC9_Mmin_reh = exp(0.75_kp*log(sqrt(3._kp)/sigmaend)+0.125_kp*log(Vend0*msigma**2/ &
                    (6._kp*Gammasigma**2*Gammaphi**2))-0.75_kp+3._kp/ &
                    8._kp*sigmaend**2+3._kp/8._kp*log(243._kp/(2._kp*Vend0))+1.5_kp*Nzero)

  end function MC9_Mmin_reh

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!        MC10         !!!!
!!!!                     !!!!
!!!!                     !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  function MC10_sigmaend_max(V,phiend0,Vparam,msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC10_sigmaend_max, Vend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Vend = V(phiend0(Vparam),Vparam)
    MC10_sigmaend_max = sqrt(2._kp*Vend)/msigma

  end function MC10_sigmaend_max


!returns the minimum allowed value for sigmaend for case 6
  function MC10_sigmaend_min(msigma,Gammaphi,Gammasigma)
    implicit none
    real(kp), intent(in) :: msigma,Gammaphi,Gammasigma
    real(kp) :: MC10_sigmaend_min

    MC10_sigmaend_min = MC9_sigmaend_max(msigma,Gammaphi,Gammasigma)

  end function MC10_sigmaend_min

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! First equivalence Hubble  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the Hubble parameter at the time of first equality between energy densities of phi and sigma
  function MC10_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    real(kp), parameter :: tolFind=tolkp
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC10_Heq = MC3_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC10_Heq

! Returns sigma at the equivalence time, as a function of Heq
  function MC10_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    real(kp) :: MC10_sigmaeq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, Heq
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC10_sigmaeq = MC3_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, &
                  sigmaend, Heq)

  end function MC10_sigmaeq

!!!!!!!!!!!!!!!!!!
!! rdec factor  !!
!!!!!!!!!!!!!!!!!!

! Energy density of photons (phi decay products) at last decay time
  function MC10_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_rhogamma_dec, Hend, rhophiend, sigmaeq, Heq
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    Hend = MC_Hend(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    rhophiend = MC_rhophi_end(V,Vpp,phiend0,Vparam,msigma,sigmaend)
    Heq = MC10_Heq(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    sigmaeq = MC10_sigmaeq(V,Vpp,phiend0,Vparam, msigma, Gammaphi, Gammasigma, sigmaend, Heq)
    MC10_rhogamma_dec = rhophiend*(Heq/Hend)**2* &
                (Gammasigma*sqrt(2._kp)/msigma)**(8._kp/3._kp)* &
                exp(2._kp-0.75_kp*sigmaeq**2-1.5_kp*Gammaphi**2/msigma**2)

  end function MC10_rhogamma_dec

! Energy density of matter (sigma) at last decay time
  function MC10_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_rhomatter_dec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC10_rhomatter_dec = 3._kp*Gammasigma**2

  end function MC10_rhomatter_dec

  function MC10_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_rdec, RhoGammaDec, RhoMatterDec
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    RhoGammaDec = MC10_rhogamma_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi,&
                    Gammasigma, sigmaend)
    RhoMatterDec = MC10_rhomatter_dec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                    Gammasigma, sigmaend)
    MC10_rdec = MC_rdec(RhoGammaDec,RhoMatterDec)

  end function MC10_rdec

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! d rho_dec / d sigma* and phi*  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns d lnrho_rad_dec / d sigmastar
  function MC10_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_dlnrho_rad_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_dlnrho_rad_dec_dsigmastar = MC6_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_dlnrho_rad_dec_dsigmastar

! Returns d lnrho_rad_dec / d phistar
  function MC10_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_dlnrho_rad_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_dlnrho_rad_dec_dphistar = MC6_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_dlnrho_rad_dec_dphistar

! Returns d^2 lnrho_rad_dec / d sigmastar ^2
  function MC10_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_rad_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_rad_dec_ddsigmastar = MC6_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_ddlnrho_rad_dec_ddsigmastar

! Returns d^2 lnrho_rad_dec / d phistar ^2
  function MC10_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_rad_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_rad_dec_ddphistar = MC6_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_ddlnrho_rad_dec_ddphistar

! Returns d^2 lnrho_rad_dec / d phistar d sigmastar
  function MC10_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_rad_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_rad_dec_dphistar_dsigmastar = MC6_ddlnrho_rad_dec_dphistar_dsigmastar(V, &
            Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
            Gammasigma, sigmaend, phistar)

  end function MC10_ddlnrho_rad_dec_dphistar_dsigmastar

! Returns d lnrho_mat_dec / d sigmastar
  function MC10_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, &
        msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_dlnrho_mat_dec_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_dlnrho_mat_dec_dsigmastar = MC6_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_dlnrho_mat_dec_dsigmastar

! Returns d lnrho_mat_dec / d phistar
  function MC10_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
        Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_dlnrho_mat_dec_dphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_dlnrho_mat_dec_dphistar = MC6_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
        InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_dlnrho_mat_dec_dphistar

! Returns d^2 lnrho_mat_dec / d sigmastar ^2
  function MC10_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_mat_dec_ddsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_mat_dec_ddsigmastar = MC6_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, &
            phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
            sigmaend, phistar)

  end function MC10_ddlnrho_mat_dec_ddsigmastar

! Returns d^2 lnrho_mat_dec / d phistar ^2
  function MC10_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, InverseVp_primitive, &
            Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_mat_dec_ddphistar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_mat_dec_ddphistar = MC6_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, &
        phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

  end function MC10_ddlnrho_mat_dec_ddphistar

! Returns d^2 lnrho_mat_dec / d phistar d sigmastar
  function MC10_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
            InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_ddlnrho_mat_dec_dphistar_dsigmastar
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    MC10_ddlnrho_mat_dec_dphistar_dsigmastar = MC6_ddlnrho_mat_dec_dphistar_dsigmastar(V, &
        Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, &
        sigmaend, phistar)

  end function MC10_ddlnrho_mat_dec_dphistar_dsigmastar

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Power Spectrum Amplitude  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Returns Pzeta
  function MC10_Pzeta(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_Pzeta
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC10_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC10_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC10_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC10_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC10_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC10_Pzeta = MC_Pzeta(Hstar, rdec,dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                           dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC10_Pzeta

! Returns the relative contribution of sigma to the total curvature power spectrum
  function MC10_Pzeta_relativecontribution_sigma(V, Vp, Vpp, phiend0, InverseVp_primitive, &
                Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_Pzeta_relativecontribution_sigma
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    real(kp) :: Hstar, rdec, dlnrho_matter_dmatterfield, &
        dlnrho_Gamma_dmatterfield, dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield, &
        sigmastar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    sigmastar = MC_inftraj_sigma(V,Vpp,phiend0,InverseVp_primitive,Vparam,phistar,msigma,sigmaend)
    Hstar = sqrt(V(phistar,Vparam)/3._kp)
    rdec = MC10_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrho_matter_dmatterfield = MC10_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dmatterfield = MC10_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_matter_dGammafield = MC10_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrho_Gamma_dGammafield = MC10_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC10_Pzeta_relativecontribution_sigma = MC_Pzeta_relativecontribution_matter(Hstar, rdec, &
                        dlnrho_matter_dmatterfield, dlnrho_Gamma_dmatterfield, &
                        dlnrho_matter_dGammafield, dlnrho_Gamma_dGammafield)

  end function MC10_Pzeta_relativecontribution_sigma

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!       fNL parameter       !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! phistar is a free parameter, not fixed by inflationary SR traj + reheating yet
  function MC10_fNL_phistarfree(V, Vp, Vpp, phiend0, InverseVp_primitive, Vparam, msigma, Gammaphi, &
          Gammasigma, sigmaend, phistar)
    implicit none
    real(kp) :: MC10_fNL_phistarfree, rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, &
        ddlnrhorad_ddphirad, ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
        dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
        ddlnrhomat_dphirad_dphimat
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend, phistar
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vp
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
        function InverseVp_primitive(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: InverseVp_primitive
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function InverseVp_primitive
    end interface

    rdec = MC10_rdec(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    dlnrhomat_dphimat = MC10_dlnrho_mat_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphimat = MC10_dlnrho_rad_dec_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhomat_dphirad = MC10_dlnrho_mat_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    dlnrhorad_dphirad = MC10_dlnrho_rad_dec_dphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphirad = MC10_ddlnrho_rad_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_ddphimat = MC10_ddlnrho_rad_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhorad_dphirad_dphimat = MC10_ddlnrho_rad_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphirad = MC10_ddlnrho_mat_dec_ddphistar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_ddphimat = MC10_ddlnrho_mat_dec_ddsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)
    ddlnrhomat_dphirad_dphimat = MC10_ddlnrho_mat_dec_dphistar_dsigmastar(V, Vp, Vpp, phiend0, &
         InverseVp_primitive, Vparam, msigma, Gammaphi, Gammasigma, sigmaend, phistar)

    MC10_fNL_phistarfree = MC_fnl(rdec, dlnrhorad_dphirad, dlnrhorad_dphimat, ddlnrhorad_ddphirad, &
                               ddlnrhorad_ddphimat, ddlnrhorad_dphirad_dphimat, dlnrhomat_dphirad, &
                               dlnrhomat_dphimat, ddlnrhomat_ddphirad, ddlnrhomat_ddphimat, &
                               ddlnrhomat_dphirad_dphimat)

  end function MC10_fNL_phistarfree

!!!!!!!!!!!!!!!!!!!!!!!!!
!! Reheating Parameter !!
!!!!!!!!!!!!!!!!!!!!!!!!!

! Returns the reheating parameter lnRrad
  function MC10_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_lnRrad
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC10_lnRrad = MC3_lnRrad(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)

  end function MC10_lnRrad

! Returns the minimum value of M so that the CMB pivot scale crosses the Hubble radius during the first inflationary period. WARNING: This function must be called with M=1 in the Vparam argument.
  function MC10_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, Gammasigma, sigmaend)
    implicit none
    real(kp) :: MC10_Mmin_reh
    real(kp), intent(in) :: msigma, Gammaphi, Gammasigma, sigmaend
    type(transfert) :: Vparam
    interface
        function V(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: V
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function V
        function Vpp(phi,otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: Vpp
           real(kp), intent(in) :: phi
           type(transfert), optional, intent(inout) :: otherparam
        end function Vpp
        function phiend0(otherparam)
           use infprec, only : kp,transfert         
           implicit none                     
           real(kp) :: phiend0
           type(transfert), optional, intent(inout) :: otherparam
        end function phiend0
    end interface

    MC10_Mmin_reh = MC3_Mmin_reh(V, Vpp, phiend0, Vparam, msigma, Gammaphi, &
                Gammasigma, sigmaend)

  end function MC10_Mmin_reh


end module curvaton

